@@ -39,13 +39,13 @@
 #include "qbtsession.h"
 #include "iconprovider.h"
 #include "fs_utils.h"
+#include "autoexpandabledialog.h"
+#include "messageboxraised.h"
 
 #include <QString>
 #include <QFile>
-#include <QInputDialog>
 #include <QUrl>
 #include <QMenu>
-#include <QMessageBox>
 #include <QTimer>
 #include <QFileDialog>
 #include <libtorrent/version.hpp>
@@ -58,11 +58,14 @@ AddNewTorrentDialog::AddNewTorrentDialog(QWidget *parent) :
   m_contentModel(0),
   m_contentDelegate(0),
   m_isMagnet(false),
+  m_hasMetadata(false),
   m_hasRenamedFile(false)
 {
   ui->setupUi(this);
+  ui->lblMetaLoading->setVisible(false);
+  ui->progMetaLoading->setVisible(false);
 
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   Preferences pref;
   ui->start_torrent_cb->setChecked(!pref.addTorrentsInPause());
   ui->save_path_combo->addItem(fsutils::toDisplayPath(pref.getSavePath()), pref.getSavePath());
@@ -78,6 +81,8 @@ AddNewTorrentDialog::AddNewTorrentDialog(QWidget *parent) :
   foreach (const QString& label, customLabels) {
     ui->label_combo->addItem(label);
   }
+  ui->label_combo->model()->sort(0);
+  ui->content_tree->header()->setSortIndicator(0, Qt::AscendingOrder);
   loadState();
   // Signal / slots
   connect(ui->adv_button, SIGNAL(clicked(bool)), SLOT(showAdvancedSettings(bool)));
@@ -97,11 +102,9 @@ AddNewTorrentDialog::~AddNewTorrentDialog()
 
 void AddNewTorrentDialog::loadState()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.beginGroup(QString::fromUtf8("AddNewTorrentDialog"));
-  QByteArray state = settings.value("treeHeaderState").toByteArray();
-  if (!state.isEmpty())
-    ui->content_tree->header()->restoreState(state);
+  m_headerState = settings.value("treeHeaderState").toByteArray();
   int width = settings.value("width", -1).toInt();
   if (width >= 0) {
     QRect geo = geometry();
@@ -113,7 +116,7 @@ void AddNewTorrentDialog::loadState()
 
 void AddNewTorrentDialog::saveState()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.beginGroup(QString::fromUtf8("AddNewTorrentDialog"));
   if (m_contentModel)
     settings.setValue("treeHeaderState", ui->content_tree->header()->saveState());
@@ -136,13 +139,23 @@ void AddNewTorrentDialog::showMagnet(const QString& link)
     dlg.exec();
 }
 
+void AddNewTorrentDialog::showEvent(QShowEvent *event) {
+  QDialog::showEvent(event);
+  Preferences pref;
+  if (!pref.AdditionDialogFront())
+    return;
+  activateWindow();
+  raise();
+}
+
+
 void AddNewTorrentDialog::showAdvancedSettings(bool show)
 {
   if (show) {
     ui->adv_button->setText(QString::fromUtf8("▲"));
     ui->settings_group->setVisible(true);
-    ui->info_group->setVisible(!m_isMagnet);
-    if (!m_isMagnet && (m_torrentInfo->num_files() > 1)) {
+    ui->info_group->setVisible(m_hasMetadata);
+    if (m_hasMetadata && (m_torrentInfo->num_files() > 1)) {
       ui->content_tree->setVisible(true);
       setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
     } else {
@@ -170,84 +183,46 @@ bool AddNewTorrentDialog::loadTorrent(const QString& torrent_path, const QString
     m_filePath = torrent_path;
 
   if (!QFile::exists(m_filePath)) {
-    QMessageBox::critical(0, tr("I/O Error"), tr("The torrent file does not exist."));
+    MessageBoxRaised::critical(0, tr("I/O Error"), tr("The torrent file does not exist."));
     return false;
   }
 
+  m_hasMetadata = true;
+
   try {
     m_torrentInfo = new torrent_info(m_filePath.toUtf8().data());
     m_hash = misc::toQString(m_torrentInfo->info_hash());
   } catch(const std::exception& e) {
-    QMessageBox::critical(0, tr("Invalid torrent"), tr("Failed to load the torrent: %1").arg(e.what()));
+    MessageBoxRaised::critical(0, tr("Invalid torrent"), tr("Failed to load the torrent: %1").arg(e.what()));
     return false;
   }
 
-  // Set dialog title
-  setWindowTitle(misc::toQStringU(m_torrentInfo->name()));
-
-  // Set torrent information
-  QString comment = misc::toQString(m_torrentInfo->comment());
-  ui->comment_lbl->setText(comment.replace('\n', ' '));
-  ui->date_lbl->setText(m_torrentInfo->creation_date() ? misc::toQString(*m_torrentInfo->creation_date()) : tr("Not available"));
-  updateDiskSpaceLabel();
-
-#if LIBTORRENT_VERSION_MINOR >= 16
-  file_storage fs = m_torrentInfo->files();
-#endif
-
-  // Populate m_filesList
-  for (int i = 0; i < m_torrentInfo->num_files(); ++i) {
-#if LIBTORRENT_VERSION_MINOR >= 16
-    m_filesPath << misc::toQStringU(fs.file_path(m_torrentInfo->file_at(i)));
-#else
-    m_filesPath << misc::toQStringU(m_torrentInfo->file_at(i).path.string());
-#endif
+  // Prevent showing the dialog if download is already present
+  if (QBtSession::instance()->getTorrentHandle(m_hash).is_valid()) {
+    MessageBoxRaised::information(0, tr("Already in download list"), tr("Torrent is already in download list. Merging trackers."), QMessageBox::Ok);
+    QBtSession::instance()->addTorrent(m_filePath, false, m_url);;
+    return false;
   }
 
-  // Prepare content tree
-  if (m_torrentInfo->num_files() > 1) {
-    m_contentModel = new TorrentContentFilterModel(this);
-    connect(m_contentModel->model(), SIGNAL(filteredFilesChanged()), SLOT(updateDiskSpaceLabel()));
-    ui->content_tree->setModel(m_contentModel);
-    ui->content_tree->hideColumn(PROGRESS);
-    m_contentDelegate = new PropListDelegate();
-    ui->content_tree->setItemDelegate(m_contentDelegate);
-    connect(ui->content_tree, SIGNAL(clicked(const QModelIndex&)), ui->content_tree, SLOT(edit(const QModelIndex&)));
-    connect(ui->content_tree, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayContentTreeMenu(const QPoint&)));
-
-    // List files in torrent
-    m_contentModel->model()->setupModelData(*m_torrentInfo);
-
-    // Expand root folder
-    ui->content_tree->setExpanded(m_contentModel->index(0, 0), true);
-    ui->content_tree->header()->setResizeMode(0, QHeaderView::Stretch);
-  } else {
-    // Update save paths (append file name to them)
-#if LIBTORRENT_VERSION_MINOR >= 16
-    QString single_file_relpath = misc::toQStringU(fs.file_path(m_torrentInfo->file_at(0)));
-#else
-    QString single_file_relpath = misc::toQStringU(m_torrentInfo->file_at(0).path.string());
-#endif
-    for (int i=0; i<ui->save_path_combo->count()-1; ++i) {
-      ui->save_path_combo->setItemText(i, fsutils::toDisplayPath(QDir(ui->save_path_combo->itemText(i)).absoluteFilePath(single_file_relpath)));
-    }
-  }
-
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
-  showAdvancedSettings(settings.value("AddNewTorrentDialog/expanded").toBool());
-  // Set dialog position
-  setdialogPosition();
-
+  setupTreeview();
   return true;
 }
 
 bool AddNewTorrentDialog::loadMagnet(const QString &magnet_uri)
 {
+  connect(QBtSession::instance(), SIGNAL(metadataReceivedHidden(const QTorrentHandle&)), SLOT(updateMetadata(const QTorrentHandle&)));
   m_isMagnet = true;
   m_url = magnet_uri;
   m_hash = misc::magnetUriToHash(m_url);
   if (m_hash.isEmpty()) {
-    QMessageBox::critical(0, tr("Invalid magnet link"), tr("This magnet link was not recognized"));
+    MessageBoxRaised::critical(0, tr("Invalid magnet link"), tr("This magnet link was not recognized"));
+    return false;
+  }
+
+  // Prevent showing the dialog if download is already present
+  if (QBtSession::instance()->getTorrentHandle(m_hash).is_valid()) {
+    MessageBoxRaised::information(0, tr("Already in download list"), tr("Magnet link is already in download list. Merging trackers."), QMessageBox::Ok);
+    QBtSession::instance()->addMagnetUri(m_url, false);
     return false;
   }
 
@@ -255,18 +230,25 @@ bool AddNewTorrentDialog::loadMagnet(const QString &magnet_uri)
   QString torrent_name = misc::magnetUriToName(m_url);
   setWindowTitle(torrent_name.isEmpty() ? tr("Magnet link") : torrent_name);
 
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   showAdvancedSettings(settings.value("AddNewTorrentDialog/expanded").toBool());
   // Set dialog position
   setdialogPosition();
 
+  Preferences pref;
+  // Override save path
+  TorrentTempData::setSavePath(m_hash, QString(QDir::tempPath() + QDir::separator() + m_hash).replace("\\", "/"));
+  HiddenData::addData(m_hash);
+  QBtSession::instance()->addMagnetUri(m_url, false);
+  setMetadataProgressIndicator(true, tr("Retrieving metadata..."));
+
   return true;
 }
 
 void AddNewTorrentDialog::saveSavePathHistory() const
 {
   QDir selected_save_path(ui->save_path_combo->itemData(ui->save_path_combo->currentIndex()).toString());
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   // Get current history
   QStringList history = settings.value("TorrentAdditionDlg/save_path_history").toStringList();
   QList<QDir> history_dirs;
@@ -304,7 +286,7 @@ void AddNewTorrentDialog::updateFileNameInSavePaths(const QString &new_filename)
 
 void AddNewTorrentDialog::updateDiskSpaceLabel()
 {
-  Q_ASSERT(!m_isMagnet);
+  Q_ASSERT(m_hasMetadata);
   // Determine torrent size
   qulonglong torrent_size = 0;
   if (m_contentModel) {
@@ -378,7 +360,7 @@ void AddNewTorrentDialog::onSavePathChanged(int index)
   relayout();
   // Remember index
   old_index = ui->save_path_combo->currentIndex();
-  if (!m_isMagnet)
+  if (m_hasMetadata)
     updateDiskSpaceLabel();
 }
 
@@ -399,17 +381,17 @@ void AddNewTorrentDialog::renameSelectedFile()
   const QModelIndex &index = selectedIndexes.first();
   // Ask for new name
   bool ok;
-  const QString new_name_last = QInputDialog::getText(this, tr("Rename the file"),
+  const QString new_name_last = AutoExpandableDialog::getText(this, tr("Rename the file"),
                                                       tr("New name:"), QLineEdit::Normal,
                                                       index.data().toString(), &ok).trimmed();
   if (ok && !new_name_last.isEmpty()) {
     if (!fsutils::isValidFileSystemName(new_name_last)) {
-      QMessageBox::warning(this, tr("The file could not be renamed"),
+      MessageBoxRaised::warning(this, tr("The file could not be renamed"),
                            tr("This file name contains forbidden characters, please choose a different one."),
                            QMessageBox::Ok);
       return;
     }
-    if (m_contentModel->getType(index) == TorrentContentModelItem::TFILE) {
+    if (m_contentModel->itemType(index) == TorrentContentModelItem::FileType) {
       // File renaming
       const int file_index = m_contentModel->getFileIndex(index);
       QString old_name = m_filesPath.at(file_index);
@@ -430,7 +412,7 @@ void AddNewTorrentDialog::renameSelectedFile()
         if (i == file_index) continue;
         if (fsutils::sameFileNames(m_filesPath.at(i), new_name)) {
           // Display error message
-          QMessageBox::warning(this, tr("The file could not be renamed"),
+          MessageBoxRaised::warning(this, tr("The file could not be renamed"),
                                tr("This name is already in use in this folder. Please use a different name."),
                                QMessageBox::Ok);
           return;
@@ -465,7 +447,7 @@ void AddNewTorrentDialog::renameSelectedFile()
 #else
         if (current_name.startsWith(new_path, Qt::CaseInsensitive)) {
 #endif
-          QMessageBox::warning(this, tr("The folder could not be renamed"),
+          MessageBoxRaised::warning(this, tr("The folder could not be renamed"),
                                tr("This name is already in use in this folder. Please use a different name."),
                                QMessageBox::Ok);
           return;
@@ -495,7 +477,7 @@ void AddNewTorrentDialog::setdialogPosition()
   qApp->processEvents();
   QPoint center(misc::screenCenter(this));
   // Adjust y
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   int y = settings.value("AddNewTorrentDialog/y", -1).toInt();
   if (y >= 0) {
     center.setY(y);
@@ -509,7 +491,7 @@ void AddNewTorrentDialog::setdialogPosition()
 
 void AddNewTorrentDialog::loadSavePathHistory()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   QDir default_save_path(Preferences().getSavePath());
   // Load save path history
   QStringList raw_path_history = settings.value("TorrentAdditionDlg/save_path_history").toStringList();
@@ -560,8 +542,11 @@ void AddNewTorrentDialog::displayContentTreeMenu(const QPoint&) {
   }
 }
 
-void AddNewTorrentDialog::on_buttonBox_accepted()
+void AddNewTorrentDialog::accept()
 {
+  if (m_isMagnet)
+    disconnect(this, SLOT(updateMetadata(const QTorrentHandle&)));
+
   Preferences pref;
   // Save Temporary data about torrent
   QString save_path = ui->save_path_combo->itemData(ui->save_path_combo->currentIndex()).toString();
@@ -570,6 +555,7 @@ void AddNewTorrentDialog::on_buttonBox_accepted()
     // TODO: Check if destination actually exists
     TorrentTempData::setSeedingMode(m_hash, true);
   }
+  pref.addTorrentsInPause(!ui->start_torrent_cb->isChecked());
 
   // Label
   const QString label = ui->label_combo->currentText();
@@ -584,20 +570,12 @@ void AddNewTorrentDialog::on_buttonBox_accepted()
   if (m_hasRenamedFile)
     TorrentTempData::setFilesPath(m_hash, m_filesPath);
 
-  // Temporary override of addInPause setting
-  bool old_addInPause = pref.addTorrentsInPause();
-  pref.addTorrentsInPause(!ui->start_torrent_cb->isChecked());
-  pref.sync();
-
   // Add torrent
   if (m_isMagnet)
-    QBtSession::instance()->addMagnetUri(m_url, false);
+    QBtSession::instance()->unhideMagnet(m_hash);
   else
     QBtSession::instance()->addTorrent(m_filePath, false, m_url);
 
-  // Restore addInPause setting
-  pref.addTorrentsInPause(old_addInPause);
-
   saveSavePathHistory();
   // Save settings
   pref.useAdditionDialog(!ui->never_show_cb->isChecked());
@@ -605,4 +583,104 @@ void AddNewTorrentDialog::on_buttonBox_accepted()
     pref.setSavePath(ui->save_path_combo->itemData(ui->save_path_combo->currentIndex()).toString());
     QBtSession::instance()->setDefaultSavePath(pref.getSavePath());
   }
+  QDialog::accept();
+}
+
+void AddNewTorrentDialog::reject() {
+  if (m_isMagnet) {
+    disconnect(this, SLOT(updateMetadata(const QTorrentHandle&)));
+    setMetadataProgressIndicator(false);
+    QBtSession::instance()->deleteTorrent(m_hash, true);
+  }
+  QDialog::reject();
+}
+
+void AddNewTorrentDialog::updateMetadata(const QTorrentHandle &h) {
+  try {
+    if (h.hash() != m_hash)
+      return;
+
+    disconnect(this, SLOT(updateMetadata(const QTorrentHandle&)));
+    Q_ASSERT(h.has_metadata());
+
+    m_torrentInfo = new torrent_info(h.get_torrent_info());
+
+    // Good to go
+    m_hasMetadata = true;
+    setMetadataProgressIndicator(true, tr("Parsing metadata..."));
+
+    // Update UI
+    setupTreeview();
+    setMetadataProgressIndicator(false, tr("Metadata retrieval complete"));
+  } catch (invalid_handle&) {
+    MessageBoxRaised::critical(0, tr("I/O Error"), ("Unknown error."));
+    setMetadataProgressIndicator(false, tr("Unknown error"));
+    return;
+  }
+}
+
+void AddNewTorrentDialog::setMetadataProgressIndicator(bool visibleIndicator, const QString &labelText) {
+  // Always show info label when waiting for metadata
+  ui->lblMetaLoading->setVisible(true);
+  ui->lblMetaLoading->setText(labelText);
+  ui->progMetaLoading->setVisible(visibleIndicator);
+}
+
+void AddNewTorrentDialog::setupTreeview() {
+  // Set dialog title
+  setWindowTitle(misc::toQStringU(m_torrentInfo->name()));
+
+  // Set torrent information
+  QString comment = misc::toQString(m_torrentInfo->comment());
+  ui->comment_lbl->setText(comment.replace('\n', ' '));
+  ui->date_lbl->setText(m_torrentInfo->creation_date() ? misc::toQString(*m_torrentInfo->creation_date()) : tr("Not available"));
+  updateDiskSpaceLabel();
+
+#if LIBTORRENT_VERSION_NUM >= 1600
+    file_storage fs = m_torrentInfo->files();
+#endif
+
+  // Populate m_filesList
+    for (int i = 0; i < m_torrentInfo->num_files(); ++i) {
+#if LIBTORRENT_VERSION_NUM >= 1600
+      m_filesPath << misc::toQStringU(fs.file_path(m_torrentInfo->file_at(i)));
+#else
+      m_filesPath << misc::toQStringU(m_torrentInfo->file_at(i).path.string());
+#endif
+  }
+
+  // Prepare content tree
+  if (fs.num_files() > 1) {
+    m_contentModel = new TorrentContentFilterModel(this);
+    connect(m_contentModel->model(), SIGNAL(filteredFilesChanged()), SLOT(updateDiskSpaceLabel()));
+    ui->content_tree->setModel(m_contentModel);
+    ui->content_tree->hideColumn(PROGRESS);
+    m_contentDelegate = new PropListDelegate();
+    ui->content_tree->setItemDelegate(m_contentDelegate);
+    connect(ui->content_tree, SIGNAL(clicked(const QModelIndex&)), ui->content_tree, SLOT(edit(const QModelIndex&)));
+    connect(ui->content_tree, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayContentTreeMenu(const QPoint&)));
+
+    // List files in torrent
+    m_contentModel->model()->setupModelData(*m_torrentInfo);
+    if (!m_headerState.isEmpty())
+      ui->content_tree->header()->restoreState(m_headerState);
+
+    // Expand root folder
+    ui->content_tree->setExpanded(m_contentModel->index(0, 0), true);
+  } else {
+    // Update save paths (append file name to them)
+#if LIBTORRENT_VERSION_NUM >= 1600
+      QString single_file_relpath = misc::toQStringU(fs.file_path(m_torrentInfo->file_at(0)));
+#else
+      QString single_file_relpath = misc::toQStringU(m_torrentInfo->file_at(0).path.string());
+#endif
+    for (int i=0; i<ui->save_path_combo->count()-1; ++i) {
+        ui->save_path_combo->setItemText(i, fsutils::toDisplayPath(QDir(ui->save_path_combo->itemText(i)).absoluteFilePath(single_file_relpath)));
+    }
+  }
+
+  QIniSettings settings;
+  showAdvancedSettings(settings.value("AddNewTorrentDialog/expanded").toBool());
+  // Set dialog position
+  setdialogPosition();
 }
