@@ -40,6 +40,7 @@
 #include "rsssettings.h"
 #endif
 #include "qinisettings.h"
+#include <zlib.h>
 
 /** Download Thread **/
 
@@ -57,6 +58,7 @@ QByteArray DownloadThread::gUncompress(Bytef *inData, size_t len) {
   }
 
   QByteArray result;
+
   z_stream strm;
   static const int CHUNK_SIZE = 1024;
   char out[CHUNK_SIZE];
@@ -71,7 +73,7 @@ QByteArray DownloadThread::gUncompress(Bytef *inData, size_t len) {
   const int windowBits = 15;
   const int ENABLE_ZLIB_GZIP = 32;
 
-  int ret = inflateInit2(&strm, windowBits|ENABLE_ZLIB_GZIP); // gzip decoding
+  int ret = inflateInit2(&strm, windowBits|ENABLE_ZLIB_GZIP ); // gzip decoding
   if (ret != Z_OK)
     return QByteArray();
 
@@ -84,12 +86,11 @@ QByteArray DownloadThread::gUncompress(Bytef *inData, size_t len) {
     Q_ASSERT(ret != Z_STREAM_ERROR); // state not clobbered
 
     switch (ret) {
-    case Z_NEED_DICT:
-    case Z_DATA_ERROR:
-    case Z_MEM_ERROR:
-      (void) inflateEnd(&strm);
-
-      return QByteArray();
+      case Z_NEED_DICT:
+      case Z_DATA_ERROR:
+      case Z_MEM_ERROR:
+        (void) inflateEnd(&strm);
+        return QByteArray();
     }
 
     result.append(out, CHUNK_SIZE - strm.avail_out);
@@ -133,8 +134,8 @@ void DownloadThread::processDlFinished(QNetworkReply* reply) {
   }
   // Success
   QTemporaryFile *tmpfile = new QTemporaryFile;
-  tmpfile->setAutoRemove(false);
   if (tmpfile->open()) {
+    tmpfile->setAutoRemove(false);
     QString filePath = tmpfile->fileName();
     qDebug("Temporary filename is: %s", qPrintable(filePath));
     if (reply->isOpen() || reply->open(QIODevice::ReadOnly)) {
@@ -152,6 +153,7 @@ void DownloadThread::processDlFinished(QNetworkReply* reply) {
       emit downloadFinished(url, filePath);
     } else {
       delete tmpfile;
+      fsutils::forceRemove(filePath);
       // Error when reading the request
       emit downloadFailure(url, tr("I/O Error"));
     }
@@ -201,16 +203,16 @@ void DownloadThread::checkDownloadSize(qint64 bytesReceived, qint64 bytesTotal)
   if (!reply) return;
   if (bytesTotal > 0) {
     // Total number of bytes is available
-    if (bytesTotal > 1048576) {
-      // More than 1MB, this is probably not a torrent file, aborting...
+    if (bytesTotal > 1048576*10) {
+      // More than 10MB, this is probably not a torrent file, aborting...
       reply->abort();
       reply->deleteLater();
     } else {
       disconnect(reply, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(checkDownloadSize(qint64,qint64)));
     }
   } else {
-    if (bytesReceived  > 1048576) {
-      // More than 1MB, this is probably not a torrent file, aborting...
+    if (bytesReceived  > 1048576*10) {
+      // More than 10MB, this is probably not a torrent file, aborting...
       reply->abort();
       reply->deleteLater();
     }
