@@ -72,6 +72,9 @@
 #include "torrentmodel.h"
 #include "executionlog.h"
 #include "iconprovider.h"
+#ifndef DISABLE_GUI
+#include "autoexpandabledialog.h"
+#endif
 #ifdef Q_WS_MAC
 #include "qmacapplication.h"
 void qt_mac_set_dock_menu(QMenu *menu);
@@ -103,7 +106,7 @@ MainWindow::MainWindow(QWidget *parent, const QStringList& torrentCmdLine) : QMa
   setWindowTitle(QString("qBittorrent %1").arg(QString::fromUtf8(VERSION)));
   displaySpeedInTitle = pref.speedInTitleBar();
   // Clean exit on log out
-  connect(static_cast<SessionApplication*>(qApp), SIGNAL(sessionIsShuttingDown()), this, SLOT(deleteBTSession()));
+  connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(shutdownCleanUp()), Qt::DirectConnection);
   // Setting icons
 #if defined(Q_WS_X11)
   if (Preferences().useSystemIconTheme())
@@ -120,6 +123,7 @@ MainWindow::MainWindow(QWidget *parent, const QStringList& torrentCmdLine) : QMa
   actionSet_global_download_limit->setIcon(QIcon(QString::fromUtf8(":/Icons/skin/download.png")));
   actionCreate_torrent->setIcon(IconProvider::instance()->getIcon("document-edit"));
   actionAbout->setIcon(IconProvider::instance()->getIcon("help-about"));
+  actionStatistics->setIcon(IconProvider::instance()->getIcon("view-statistics"));
   actionBugReport->setIcon(IconProvider::instance()->getIcon("tools-report-bug"));
   actionDecreasePriority->setIcon(IconProvider::instance()->getIcon("go-down"));
   actionDelete->setIcon(IconProvider::instance()->getIcon("list-remove"));
@@ -145,6 +149,8 @@ MainWindow::MainWindow(QWidget *parent, const QStringList& torrentCmdLine) : QMa
   QMenu *lockMenu = new QMenu(this);
   QAction *defineUiLockPasswdAct = lockMenu->addAction(tr("Set the password..."));
   connect(defineUiLockPasswdAct, SIGNAL(triggered()), this, SLOT(defineUILockPassword()));
+  QAction *clearUiLockPasswdAct = lockMenu->addAction(tr("Clear the password"));
+  connect(clearUiLockPasswdAct, SIGNAL(triggered()), this, SLOT(clearUILockPassword()));
   actionLock_qBittorrent->setMenu(lockMenu);
   // Creating Bittorrent session
   connect(QBtSession::instance(), SIGNAL(fullDiskError(QTorrentHandle, QString)), this, SLOT(fullDiskError(QTorrentHandle, QString)));
@@ -205,6 +211,15 @@ MainWindow::MainWindow(QWidget *parent, const QStringList& torrentCmdLine) : QMa
   connect(actionToggleVisibility, SIGNAL(triggered()), this, SLOT(toggleVisibility()));
   connect(actionMinimize, SIGNAL(triggered()), SLOT(minimizeWindow()));
 
+#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+  programUpdateTimer.setInterval(60*60*1000);
+  programUpdateTimer.setSingleShot(true);
+  connect(&programUpdateTimer, SIGNAL(timeout()), SLOT(checkProgramUpdate()));
+  connect(actionCheck_for_updates, SIGNAL(triggered()), SLOT(checkProgramUpdate()));
+#else
+  actionCheck_for_updates->setVisible(false);
+#endif
+
   m_pwr = new PowerManagement(this);
   preventTimer = new QTimer(this);
   connect(preventTimer, SIGNAL(timeout()), SLOT(checkForActiveTorrents()));
@@ -260,14 +275,8 @@ MainWindow::MainWindow(QWidget *parent, const QStringList& torrentCmdLine) : QMa
   // Load Window state and sizes
   readSettings();
 
-  if (!ui_locked) {
-    if (pref.startMinimized() && systrayIcon) {
-      show();
-      minimizeWindow();
-      // XXX: Using showMinimized() makes it impossible to restore
-      // the window if "Minimize to systray" is enabled.
-      //showMinimized();
-    } else {
+  if (systrayIcon) {
+    if (!(pref.startMinimized() || ui_locked)) {
       show();
       activateWindow();
       raise();
@@ -310,38 +319,28 @@ MainWindow::MainWindow(QWidget *parent, const QStringList& torrentCmdLine) : QMa
 #ifdef Q_WS_MAC
   qt_mac_set_dock_menu(getTrayIconMenu());
 #endif
-#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
-  // Check for update
-  if (pref.isUpdateCheckEnabled()) {
-    ProgramUpdater *updater = new ProgramUpdater(this);
-    connect(updater, SIGNAL(updateCheckFinished(bool, QString)), SLOT(handleUpdateCheckFinished(bool, QString)));
-    updater->checkForUpdates();
-  }
-#endif
 
   // Make sure the Window is visible if we don't have a tray icon
-  if (!systrayIcon && isHidden()) {
-    show();
-    activateWindow();
-    raise();
+  if (!systrayIcon) {
+    if (pref.startMinimized()) {
+      showMinimized();
+    } else {
+      show();
+      activateWindow();
+      raise();
+    }
   }
 }
 
-void MainWindow::deleteBTSession() {
-  guiUpdater->stop();
-  status_bar->stopTimer();
-  QBtSession::drop();
-  m_pwr->setActivityState(false);
-  // Save window size, columns size
-  writeSettings();
-  // Accept exit
-  qApp->exit();
-}
-
-// Destructor
-MainWindow::~MainWindow() {
+void MainWindow::shutdownCleanUp() {
   qDebug("GUI destruction");
   hide();
+  guiUpdater->stop();
+  status_bar->stopTimer();
+  m_pwr->setActivityState(false);
+  QBtSession::drop();
+  // Save window size, columns size
+  writeSettings();
 #ifdef Q_WS_MAC
   // Workaround to avoid bug http://bugreports.qt.nokia.com/browse/QTBUG-7305
   setUnifiedTitleAndToolBarOnMac(false);
@@ -360,6 +359,8 @@ MainWindow::~MainWindow() {
     delete m_executionLog;
   if (aboutDlg)
     delete aboutDlg;
+  if (statsDlg)
+    delete statsDlg;
   if (options)
     delete options;
   if (downloadFromURLDialog)
@@ -372,15 +373,12 @@ MainWindow::~MainWindow() {
   delete properties;
   delete hSplitter;
   delete vSplitter;
-  if (systrayCreator) {
+  if (systrayCreator)
     delete systrayCreator;
-  }
-  if (systrayIcon) {
+  if (systrayIcon)
     delete systrayIcon;
-  }
-  if (myTrayIconMenu) {
+  if (myTrayIconMenu)
     delete myTrayIconMenu;
-  }
   delete tabs;
   // Keyboard shortcuts
   delete switchSearchShortcut;
@@ -388,18 +386,15 @@ MainWindow::~MainWindow() {
   delete switchTransferShortcut;
   delete switchRSSShortcut;
   IconProvider::drop();
-  // Delete QBtSession::instance() object
-  m_pwr->setActivityState(false);
-  qDebug("Deleting QBtSession::instance()");
-  QBtSession::drop();
-  qDebug("Exiting GUI destructor...");
+  Preferences().sync();
+  qDebug("Finished GUI destruction");
 }
 
 void MainWindow::defineUILockPassword() {
   QString old_pass_md5 = Preferences().getUILockPasswordMD5();
   if (old_pass_md5.isNull()) old_pass_md5 = "";
   bool ok = false;
-  QString new_clear_password = QInputDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, old_pass_md5, &ok);
+  QString new_clear_password = AutoExpandableDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, old_pass_md5, &ok);
   if (ok) {
     new_clear_password = new_clear_password.trimmed();
     if (new_clear_password.size() < 3) {
@@ -413,13 +408,19 @@ void MainWindow::defineUILockPassword() {
   }
 }
 
+void MainWindow::clearUILockPassword() {
+  QMessageBox::StandardButton answer = QMessageBox::question(this, tr("Clear the password"), tr("Are you sure you want to clear the password?"), QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
+  if (answer == QMessageBox::Yes)
+    Preferences().clearUILockPassword();
+}
+
 void MainWindow::on_actionLock_qBittorrent_triggered() {
   Preferences pref;
   // Check if there is a password
   if (pref.getUILockPasswordMD5().isEmpty()) {
     // Ask for a password
     bool ok = false;
-    QString clear_password = QInputDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, "", &ok);
+    QString clear_password = AutoExpandableDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, "", &ok);
     if (!ok) return;
     pref.setUILockPassword(clear_password);
   }
@@ -491,7 +492,7 @@ void MainWindow::tab_changed(int new_tab) {
 }
 
 void MainWindow::writeSettings() {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.beginGroup(QString::fromUtf8("MainWindow"));
   settings.setValue("geometry", saveGeometry());
   // Splitter size
@@ -501,7 +502,7 @@ void MainWindow::writeSettings() {
 }
 
 void MainWindow::readSettings() {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.beginGroup(QString::fromUtf8("MainWindow"));
   if (settings.contains("geometry")) {
     if (restoreGeometry(settings.value("geometry").toByteArray()))
@@ -542,12 +543,13 @@ void MainWindow::finishedTorrent(const QTorrentHandle& h) const {
 // Notification when disk is full
 void MainWindow::fullDiskError(const QTorrentHandle& h, QString msg) const {
   if (!h.is_valid()) return;
-  showNotificationBaloon(tr("I/O Error", "i.e: Input/Output Error"), tr("An I/O error occured for torrent %1.\n Reason: %2", "e.g: An error occured for torrent xxx.avi.\n Reason: disk is full.").arg(h.name()).arg(msg));
+  showNotificationBaloon(tr("I/O Error", "i.e: Input/Output Error"), tr("An I/O error occurred for torrent %1.\n Reason: %2", "e.g: An error occurred for torrent xxx.avi.\n Reason: disk is full.").arg(h.name()).arg(msg));
 }
 
 void MainWindow::createKeyboardShortcuts() {
   actionCreate_torrent->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+N")));
   actionOpen->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+O")));
+  actionDownload_from_URL->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+Shift+O")));
   actionExit->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+Q")));
   switchTransferShortcut = new QShortcut(QKeySequence("Alt+1"), this);
   connect(switchTransferShortcut, SIGNAL(activated()), this, SLOT(displayTransferTab()));
@@ -559,11 +561,6 @@ void MainWindow::createKeyboardShortcuts() {
   connect(switchRSSShortcut, SIGNAL(activated()), this, SLOT(displayRSSTab()));
   actionDocumentation->setShortcut(QKeySequence("F1"));
   actionOptions->setShortcut(QKeySequence(QString::fromUtf8("Alt+O")));
-#ifdef Q_WS_MAC
-  actionDelete->setShortcut(QKeySequence("Ctrl+Backspace"));
-#else
-  actionDelete->setShortcut(QKeySequence(QString::fromUtf8("Del")));
-#endif
   actionStart->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+S")));
   actionStart_All->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+Shift+S")));
   actionPause->setShortcut(QKeySequence(QString::fromUtf8("Ctrl+P")));
@@ -626,7 +623,7 @@ void MainWindow::handleDownloadFromUrlFailure(QString url, QString reason) const
 void MainWindow::on_actionSet_global_upload_limit_triggered() {
   qDebug("actionSet_global_upload_limit_triggered");
   bool ok;
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
     int cur_limit = QBtSession::instance()->getSession()->settings().upload_rate_limit;
 #else
     int cur_limit = QBtSession::instance()->getSession()->upload_rate_limit();
@@ -645,7 +642,7 @@ void MainWindow::on_actionSet_global_upload_limit_triggered() {
 void MainWindow::on_actionSet_global_download_limit_triggered() {
   qDebug("actionSet_global_download_limit_triggered");
   bool ok;
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
     int cur_limit = QBtSession::instance()->getSession()->settings().download_rate_limit;
 #else
     int cur_limit = QBtSession::instance()->getSession()->download_rate_limit();
@@ -689,7 +686,7 @@ void MainWindow::setTabText(int index, QString text) const {
 
 bool MainWindow::unlockUI() {
   bool ok = false;
-  QString clear_password = QInputDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, "", &ok);
+  QString clear_password = AutoExpandableDialog::getText(this, tr("UI lock password"), tr("Please type the UI lock password:"), QLineEdit::Password, "", &ok);
   if (!ok) return false;
   Preferences pref;
   QString real_pass_md5 = pref.getUILockPasswordMD5();
@@ -745,6 +742,13 @@ void MainWindow::on_actionAbout_triggered() {
   }
 }
 
+void MainWindow::on_actionStatistics_triggered() {
+  if (statsDlg)
+    statsDlg->setFocus();
+  else
+    statsDlg = new StatsDialog(this);
+}
+
 void MainWindow::showEvent(QShowEvent *e) {
   qDebug("** Show Event **");
 
@@ -927,7 +931,7 @@ void MainWindow::dragEnterEvent(QDragEnterEvent *event) {
 // torrents to download list
 void MainWindow::on_actionOpen_triggered() {
   Preferences pref;
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   // Open File Open Dialog
   // Note: it is possible to select more than one file
   const QStringList pathsList = QFileDialog::getOpenFileNames(0,
@@ -1105,6 +1109,13 @@ void MainWindow::loadPreferences(bool configure_session) {
   if (configure_session)
     QBtSession::instance()->configureSession();
 
+#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+  if (pref.isUpdateCheckEnabled())
+    checkProgramUpdate();
+  else
+    programUpdateTimer.stop();
+#endif
+
   qDebug("GUI settings loaded");
 }
 
@@ -1132,16 +1143,16 @@ void MainWindow::updateGUI() {
     html += "qBittorrent";
     html += "</div>";
     html += "<div style='vertical-align: baseline; height: 18px;'>";
-    html += "<img src=':/Icons/skin/download.png'/>&nbsp;"+tr("DL speed: %1 KiB/s", "e.g: Download speed: 10 KiB/s").arg(QString::number(QBtSession::instance()->getPayloadDownloadRate()/1024., 'f', 1));
+    html += "<img src=':/Icons/skin/download.png'/>&nbsp;"+tr("DL speed: %1 KiB/s", "e.g: Download speed: 10 KiB/s").arg(misc::accurateDoubleToString(QBtSession::instance()->getPayloadDownloadRate()/1024., 1));
     html += "</div>";
     html += "<div style='vertical-align: baseline; height: 18px;'>";
-    html += "<img src=':/Icons/skin/seeding.png'/>&nbsp;"+tr("UP speed: %1 KiB/s", "e.g: Upload speed: 10 KiB/s").arg(QString::number(QBtSession::instance()->getPayloadUploadRate()/1024., 'f', 1));
+    html += "<img src=':/Icons/skin/seeding.png'/>&nbsp;"+tr("UP speed: %1 KiB/s", "e.g: Upload speed: 10 KiB/s").arg(misc::accurateDoubleToString(QBtSession::instance()->getPayloadUploadRate()/1024., 1));
     html += "</div>";
 #else
     // OSes such as Windows do not support html here
-    QString html =tr("DL speed: %1 KiB/s", "e.g: Download speed: 10 KiB/s").arg(QString::number(QBtSession::instance()->getPayloadDownloadRate()/1024., 'f', 1));
+    QString html =tr("DL speed: %1 KiB/s", "e.g: Download speed: 10 KiB/s").arg(misc::accurateDoubleToString(QBtSession::instance()->getPayloadDownloadRate()/1024., 1));
     html += "\n";
-    html += tr("UP speed: %1 KiB/s", "e.g: Upload speed: 10 KiB/s").arg(QString::number(QBtSession::instance()->getPayloadUploadRate()/1024., 'f', 1));
+    html += tr("UP speed: %1 KiB/s", "e.g: Upload speed: 10 KiB/s").arg(misc::accurateDoubleToString(QBtSession::instance()->getPayloadUploadRate()/1024., 1));
 #endif
     systrayIcon->setToolTip(html); // tray icon
   }
@@ -1156,15 +1167,22 @@ void MainWindow::showNotificationBaloon(QString title, QString msg) const {
   org::freedesktop::Notifications notifications("org.freedesktop.Notifications",
                                                 "/org/freedesktop/Notifications",
                                                 QDBusConnection::sessionBus());
-  if (notifications.isValid()) {
-    QVariantMap hints;
-    hints["desktop-entry"] = "qBittorrent";
-    QDBusPendingReply<uint> reply = notifications.Notify("qBittorrent", 0, "qbittorrent", title,
-                                                         msg, QStringList(), hints, -1);
-    reply.waitForFinished();
-    if (!reply.isError())
-      return;
-  }
+  // Testing for 'notifications.isValid()' isn't helpful here.
+  // If the notification daemon is configured to run 'as needed'
+  // the above check can be false if the daemon wasn't started
+  // by another application. In this case DBus will be able to
+  // start the notification daemon and complete our request. Such
+  // a daemon is xfce4-notifyd, DBus autostarts it and after
+  // some inactivity shuts it down. Other DEs, like GNOME, choose
+  // to start their daemons at the session startup and have it sit
+  // idling for the whole session.
+  QVariantMap hints;
+  hints["desktop-entry"] = "qBittorrent";
+  QDBusPendingReply<uint> reply = notifications.Notify("qBittorrent", 0, "qbittorrent", title,
+                                                       msg, QStringList(), hints, -1);
+  reply.waitForFinished();
+  if (!reply.isError())
+    return;
 #endif
   if (systrayIcon && QSystemTrayIcon::supportsMessages())
     systrayIcon->showMessage(title, msg, QSystemTrayIcon::Information, TIME_TRAY_BALLOON);
@@ -1184,13 +1202,20 @@ void MainWindow::downloadFromURLList(const QStringList& url_list) {
       qDebug("Converting bc link to magnet link");
       url = misc::bcLinkToMagnet(url);
     }
+    if ((url.size() == 40 && !url.contains(QRegExp("[^0-9A-Fa-f]")))
+        || (url.size() == 32 && !url.contains(QRegExp("[^2-7A-Za-z]")))) {
+      url = "magnet:?xt=urn:btih:" + url;
+    }
     if (url.startsWith("magnet:", Qt::CaseInsensitive)) {
       if (useTorrentAdditionDialog)
         AddNewTorrentDialog::showMagnet(url);
       else
         QBtSession::instance()->addMagnetUri(url);
-    } else
+    }
+    else if (url.startsWith("http://", Qt::CaseInsensitive) || url.startsWith("https://", Qt::CaseInsensitive)
+             || url.startsWith("ftp://", Qt::CaseInsensitive)) {
       QBtSession::instance()->downloadFromUrl(url);
+    }
   }
 }
 
@@ -1328,29 +1353,31 @@ void MainWindow::on_actionDownload_from_URL_triggered() {
 
 #if defined(Q_WS_WIN) || defined(Q_WS_MAC)
 
-void MainWindow::handleUpdateCheckFinished(bool update_available, QString new_version)
+void MainWindow::handleUpdateCheckFinished(bool update_available, QString new_version, bool invokedByUser)
 {
+  QMessageBox::StandardButton answer = QMessageBox::Yes;
   if (update_available) {
-    if (QMessageBox::question(this, tr("A newer version is available"),
-                             tr("A newer version of qBittorrent is available on Sourceforge.\nWould you like to update qBittorrent to version %1?").arg(new_version),
-                             QMessageBox::Yes|QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
+    answer = QMessageBox::question(this, tr("A new version is available"),
+                                   tr("A new version of qBittorrent is available on Sourceforge.\nWould you like to update qBittorrent to version %1?").arg(new_version),
+                                   QMessageBox::Yes|QMessageBox::No, QMessageBox::Yes);
+    if (answer == QMessageBox::Yes) {
       // The user want to update, let's download the update
       ProgramUpdater* updater = dynamic_cast<ProgramUpdater*>(sender());
-      connect(updater, SIGNAL(updateInstallFinished(QString)), SLOT(handleUpdateInstalled(QString)));
       updater->updateProgram();
-      return;
     }
   }
-  sender()->deleteLater();
-}
-
-void MainWindow::handleUpdateInstalled(QString error_msg)
-{
-  if (!error_msg.isEmpty()) {
-    QMessageBox::critical(this, tr("Impossible to update qBittorrent"), tr("qBittorrent failed to update, reason: %1").arg(error_msg));
+  else if (invokedByUser) {
+    QMessageBox::information(this, tr("There isn't a new version available"),
+                             tr("There isn't a new version of qBittorrent available on Sourceforge"));
   }
+  sender()->deleteLater();
+  actionCheck_for_updates->setEnabled(true);
+  actionCheck_for_updates->setText(tr("Check for updates"));
+  actionCheck_for_updates->setToolTip(tr("Check for program updates"));
+  // Don't bother the user again in this session if he chose to ignore the update
+  if (Preferences().isUpdateCheckEnabled() && answer == QMessageBox::Yes)
+    programUpdateTimer.start();
 }
-
 #endif
 
 void MainWindow::on_actionDonate_money_triggered()
@@ -1403,11 +1430,7 @@ void MainWindow::on_actionAutoShutdown_system_toggled(bool enabled)
 
 void MainWindow::checkForActiveTorrents()
 {
-  const TorrentStatusReport report = transferList->getSourceModel()->getTorrentStatusReport();
-  if (report.nb_active > 0) // Active torrents are present; prevent system from suspend
-    m_pwr->setActivityState(true);
-  else
-    m_pwr->setActivityState(false);
+  m_pwr->setActivityState(transferList->getSourceModel()->inhibitSystem());
 }
 
 QIcon MainWindow::getSystrayIcon() const
@@ -1436,3 +1459,16 @@ QIcon MainWindow::getSystrayIcon() const
   }
   return icon;
 }
+
+#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+void MainWindow::checkProgramUpdate() {
+  programUpdateTimer.stop(); // If the user had clicked the menu item
+  actionCheck_for_updates->setEnabled(false);
+  actionCheck_for_updates->setText(tr("Checking for updates..."));
+  actionCheck_for_updates->setToolTip(tr("Already checking for program updates in the background"));
+  bool invokedByUser = actionCheck_for_updates == qobject_cast<QAction*>(sender());
+  ProgramUpdater *updater = new ProgramUpdater(this, invokedByUser);
+  connect(updater, SIGNAL(updateCheckFinished(bool, QString, bool)), SLOT(handleUpdateCheckFinished(bool, QString, bool)));
+  updater->checkForUpdates();
+}
+#endif
