@@ -107,11 +107,12 @@ namespace misc
   QList<int> intListfromStringList(const QStringList &l);
   QList<bool> boolListfromStringList(const QStringList &l);
 
-#if LIBTORRENT_VERSION_MINOR < 16
+#if LIBTORRENT_VERSION_NUM < 1600
   QString toQString(const boost::posix_time::ptime& boostDate);
 #else
   QString toQString(time_t t);
 #endif
+  QString accurateDoubleToString(const double &n, const int &precision);
 
 #ifndef DISABLE_GUI
   bool naturalSort(QString left, QString right, bool& result);
