@@ -30,7 +30,6 @@
 #include <QTextStream>
 #include <QFileDialog>
 #include <QMessageBox>
-#include <QInputDialog>
 #include <QSystemTrayIcon>
 #include <QApplication>
 #include <QDialogButtonBox>
@@ -40,8 +39,6 @@
 #include <QDesktopServices>
 
 #include <libtorrent/version.hpp>
-#include <time.h>
-#include <stdlib.h>
 
 #include "options_imp.h"
 #include "preferences.h"
@@ -122,9 +119,10 @@ options_imp::options_imp(QWidget *parent):
 #endif
 
 #ifndef Q_WS_WIN
+  checkStartup->setVisible(false);
   groupFileAssociation->setVisible(false);
 #endif
-#if LIBTORRENT_VERSION_MINOR < 16
+#if LIBTORRENT_VERSION_NUM < 1600
   checkAnonymousMode->setVisible(false);
   label_anonymous->setVisible(false);
 #endif
@@ -132,6 +130,7 @@ options_imp::options_imp(QWidget *parent):
   // Connect signals / slots
   connect(comboProxyType, SIGNAL(currentIndexChanged(int)),this, SLOT(enableProxy(int)));
   connect(checkAnonymousMode, SIGNAL(toggled(bool)), this, SLOT(toggleAnonymousMode(bool)));
+  connect(checkRandomPort, SIGNAL(toggled(bool)), spinPort, SLOT(setDisabled(bool)));
 
   // Apply button is activated when a value is changed
   // General tab
@@ -141,6 +140,9 @@ options_imp::options_imp(QWidget *parent):
   connect(checkCloseToSystray, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkMinimizeToSysTray, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkStartMinimized, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
+#ifdef Q_WS_WIN
+  connect(checkStartup, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
+#endif
   connect(checkShowSplash, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkProgramExitConfirm, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkPreventFromSuspend, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
@@ -159,9 +161,12 @@ options_imp::options_imp(QWidget *parent):
   connect(checkAppendqB, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkPreallocateAll, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkAdditionDialog, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
+  connect(checkAdditionDialogFront, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkStartPaused, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkExportDir, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
+  connect(checkExportDirFin, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(textExportDir, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
+  connect(textExportDirFin, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
   connect(actionTorrentDlOnDblClBox, SIGNAL(currentIndexChanged(int)), this, SLOT(enableApplyButton()));
   connect(actionTorrentFnOnDblClBox, SIGNAL(currentIndexChanged(int)), this, SLOT(enableApplyButton()));
   connect(checkTempFolder, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
@@ -178,6 +183,7 @@ options_imp::options_imp(QWidget *parent):
   connect(autoRun_txt, SIGNAL(textChanged(QString)), this, SLOT(enableApplyButton()));
   // Connection tab
   connect(spinPort, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
+  connect(checkRandomPort, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkUPnP, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkUploadLimit, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
   connect(checkDownloadLimit, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
@@ -202,7 +208,7 @@ options_imp::options_imp(QWidget *parent):
   connect(spinMaxUploads, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
   connect(spinMaxUploadsPerTorrent, SIGNAL(valueChanged(QString)), this, SLOT(enableApplyButton()));
   connect(checkDHT, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
   connect(checkAnonymousMode, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
 #endif
   connect(checkPeX, SIGNAL(toggled(bool)), this, SLOT(enableApplyButton()));
@@ -248,7 +254,7 @@ options_imp::options_imp(QWidget *parent):
   applyButton->setEnabled(false);
   // Tab selection mecanism
   connect(tabSelection, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)));
-#if LIBTORRENT_VERSION_MINOR < 16
+#if LIBTORRENT_VERSION_NUM < 1600
   checkuTP->setVisible(false);
   checkLimituTPConnections->setVisible(false);
 #endif
@@ -261,7 +267,7 @@ options_imp::options_imp(QWidget *parent):
 
   // Adapt size
   show();
-  loadWindowState();
+  loadWindowState();  
 }
 
 void options_imp::initializeLanguageCombo()
@@ -272,9 +278,8 @@ void options_imp::initializeLanguageCombo()
   foreach (QString lang_file, lang_files) {
     QString localeStr = lang_file.mid(12); // remove "qbittorrent_"
     localeStr.chop(3); // Remove ".qm"
-    QLocale locale(localeStr);
-    const QString country = locale.name().split("_").last().toLower();
-    QString language_name = languageToLocalizedString(locale.language(), country);
+    QLocale locale(localeStr);    
+    QString language_name = languageToLocalizedString(locale);
     comboI18n->addItem(/*QIcon(":/Icons/flags/"+country+".png"), */language_name, locale.name());
     qDebug() << "Supported locale:" << locale.name();
   }
@@ -296,10 +301,11 @@ void options_imp::changePage(QListWidgetItem *current, QListWidgetItem *previous
 }
 
 void options_imp::loadWindowState() {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   resize(settings.value(QString::fromUtf8("Preferences/State/size"), sizeFittingScreen()).toSize());
   QPoint p = settings.value(QString::fromUtf8("Preferences/State/pos"), QPoint()).toPoint();
-  if (!p.isNull())
+  QRect scr_rect = qApp->desktop()->screenGeometry();
+  if (!p.isNull() && scr_rect.contains(p))
     move(p);
   // Load slider size
   const QStringList sizes_str = settings.value("Preferences/State/hSplitterSizes", QStringList()).toStringList();
@@ -316,7 +322,7 @@ void options_imp::loadWindowState() {
 }
 
 void options_imp::saveWindowState() const {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.setValue(QString::fromUtf8("Preferences/State/size"), size());
   settings.setValue(QString::fromUtf8("Preferences/State/pos"), pos());
   // Splitter size
@@ -355,7 +361,7 @@ void options_imp::saveOptions() {
     if (translator->load(QString::fromUtf8(":/lang/qbittorrent_") + locale)) {
       qDebug("%s locale recognized, using translation.", qPrintable(locale));
     }else{
-      qDebug("%s locale unrecognized, using default (en_GB).", qPrintable(locale));
+      qDebug("%s locale unrecognized, using default (en).", qPrintable(locale));
     }
     qApp->installTranslator(translator);
   }
@@ -372,6 +378,7 @@ void options_imp::saveOptions() {
   pref.setConfirmOnExit(checkProgramExitConfirm->isChecked());
   pref.setPreventFromSuspend(preventFromSuspend());
 #ifdef Q_WS_WIN
+  pref.setStartup(Startup());
   // Windows: file association settings
   Preferences::setTorrentFileAssoc(checkAssociateTorrents->isChecked());
   Preferences::setMagnetLinkAssoc(checkAssociateMagnetLinks->isChecked());
@@ -394,14 +401,18 @@ void options_imp::saveOptions() {
   pref.useIncompleteFilesExtension(checkAppendqB->isChecked());
   pref.preAllocateAllFiles(preAllocateAllFiles());
   pref.useAdditionDialog(useAdditionDialog());
+  pref.AdditionDialogFront(checkAdditionDialogFront->isChecked());
   pref.addTorrentsInPause(addTorrentsInPause());
   ScanFoldersModel::instance()->makePersistent();
   addedScanDirs.clear();
-  QString export_dir = getExportDir();
+  QString export_dir = getTorrentExportDir();
+  QString export_dir_fin = getFinishedTorrentExportDir();
 #if defined(Q_WS_WIN) || defined(Q_OS_OS2)
+  export_dir_fin.replace("\\", "/");
   export_dir.replace("\\", "/");
 #endif
-  pref.setExportDir(export_dir);
+  pref.setTorrentExportDir(export_dir);
+  pref.setFinishedTorrentExportDir(export_dir_fin);
   pref.setMailNotificationEnabled(groupMailNotification->isChecked());
   pref.setMailNotificationEmail(dest_email_txt->text());
   pref.setMailNotificationSMTP(smtp_server_txt->text());
@@ -416,6 +427,7 @@ void options_imp::saveOptions() {
   // End Downloads preferences
   // Connection preferences
   pref.setSessionPort(getPort());
+  pref.setRandomPort(checkRandomPort->isChecked());
   pref.setUPnPEnabled(isUPnPEnabled());
   const QPair<int, int> down_up_limit = getGlobalBandwidthLimits();
   pref.setGlobalDownloadLimit(down_up_limit.first);
@@ -448,7 +460,7 @@ void options_imp::saveOptions() {
   pref.setDHTPort(getDHTPort());
   pref.setLSDEnabled(isLSDEnabled());
   pref.setEncryptionSetting(getEncryptionSetting());
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
   pref.enableAnonymousMode(checkAnonymousMode->isChecked());
 #endif
   pref.setGlobalMaxRatio(getMaxRatio());
@@ -544,6 +556,7 @@ void options_imp::loadOptions() {
   checkProgramExitConfirm->setChecked(pref.confirmOnExit());
   checkPreventFromSuspend->setChecked(pref.preventFromSuspend());
 #ifdef Q_WS_WIN
+  checkStartup->setChecked(pref.Startup());
   // Windows: file association settings
   checkAssociateTorrents->setChecked(Preferences::isTorrentFileAssocSet());
   checkAssociateMagnetLinks->setChecked(Preferences::isMagnetLinkAssocSet());
@@ -570,9 +583,10 @@ void options_imp::loadOptions() {
   checkAppendqB->setChecked(pref.useIncompleteFilesExtension());
   checkPreallocateAll->setChecked(pref.preAllocateAllFiles());
   checkAdditionDialog->setChecked(pref.useAdditionDialog());
+  checkAdditionDialogFront->setChecked(pref.AdditionDialogFront());
   checkStartPaused->setChecked(pref.addTorrentsInPause());
 
-  strValue = pref.getExportDir();
+  strValue = pref.getTorrentExportDir();
   if (strValue.isEmpty()) {
     // Disable
     checkExportDir->setChecked(false);
@@ -584,6 +598,19 @@ void options_imp::loadOptions() {
 #endif
     textExportDir->setText(strValue);
   }
+
+  strValue = pref.getFinishedTorrentExportDir();
+  if (strValue.isEmpty()) {
+    // Disable
+    checkExportDirFin->setChecked(false);
+  } else {
+    // enable
+    checkExportDirFin->setChecked(true);
+#if defined(Q_WS_WIN) || defined(Q_OS_OS2)
+    strValue.replace("/", "\\");
+#endif
+    textExportDirFin->setText(strValue);
+  }
   groupMailNotification->setChecked(pref.isMailNotificationEnabled());
   dest_email_txt->setText(pref.getMailNotificationEmail());
   smtp_server_txt->setText(pref.getMailNotificationSMTP());
@@ -605,6 +632,8 @@ void options_imp::loadOptions() {
   // Connection preferences
   spinPort->setValue(pref.getSessionPort());
   checkUPnP->setChecked(pref.isUPnPEnabled());
+  checkRandomPort->setChecked(pref.useRandomPort());
+  spinPort->setDisabled(checkRandomPort->isChecked());
   intValue = pref.getGlobalDownloadLimit();
   if (intValue > 0) {
     // Enabled
@@ -717,7 +746,7 @@ void options_imp::loadOptions() {
   checkPeX->setChecked(pref.isPeXEnabled());
   checkLSD->setChecked(pref.isLSDEnabled());
   comboEncryption->setCurrentIndex(pref.getEncryptionSetting());
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
   checkAnonymousMode->setChecked(pref.isAnonymousModeEnabled());
   /* make sure ui matches options */
   toggleAnonymousMode(checkAnonymousMode->isChecked());
@@ -766,9 +795,7 @@ void options_imp::loadOptions() {
   domainNameTxt->setText(pref.getDynDomainName());
   DNSUsernameTxt->setText(pref.getDynDNSUsername());
   DNSPasswordTxt->setText(pref.getDynDNSPassword());
-  // End Web UI
-  // Random stuff
-  srand(time(0));
+  // End Web UI  
 }
 
 // return min & max ports
@@ -838,7 +865,6 @@ QPair<int,int> options_imp::getGlobalBandwidthLimits() const {
 }
 
 bool options_imp::startMinimized() const {
-  if (checkStartMinimized->isChecked()) return true;
   return checkStartMinimized->isChecked();
 }
 
@@ -914,6 +940,10 @@ int options_imp::getMaxUploadsPerTorrent() const {
 
 void options_imp::on_buttonBox_accepted() {
   if (applyButton->isEnabled()) {
+    if (!schedTimesOk()) {
+      tabSelection->setCurrentRow(TAB_SPEED);
+      return;
+    }
     saveOptions();
     applyButton->setEnabled(false);
     this->hide();
@@ -925,6 +955,10 @@ void options_imp::on_buttonBox_accepted() {
 
 void options_imp::applySettings(QAbstractButton* button) {
   if (button == applyButton) {
+    if (!schedTimesOk()) {
+      tabSelection->setCurrentRow(TAB_SPEED);
+      return;
+    }
     saveOptions();
     emit status_changed();
   }
@@ -978,6 +1012,12 @@ bool options_imp::isSlashScreenDisabled() const {
   return !checkShowSplash->isChecked();
 }
 
+#ifdef Q_WS_WIN
+bool options_imp::Startup() const {
+  return checkStartup->isChecked();
+}
+#endif
+
 bool options_imp::preventFromSuspend() const {
   return checkPreventFromSuspend->isChecked();
 }
@@ -1040,12 +1080,18 @@ void options_imp::setLocale(const QString &localeStr) {
   comboI18n->setCurrentIndex(index);
 }
 
-QString options_imp::getExportDir() const {
+QString options_imp::getTorrentExportDir() const {
   if (checkExportDir->isChecked())
     return fsutils::expandPath(textExportDir->text());
   return QString();
 }
 
+QString options_imp::getFinishedTorrentExportDir() const {
+  if (checkExportDirFin->isChecked())
+    return fsutils::expandPath(textExportDirFin->text());
+  return QString();
+}
+
 // Return action on double-click on a downloading torrent set in options
 int options_imp::getActionOnDblClOnTorrentDl() const {
   if (actionTorrentDlOnDblClBox->currentIndex() < 1)
@@ -1099,21 +1145,28 @@ void options_imp::handleScanFolderViewSelectionChanged() {
   removeScanFolderButton->setEnabled(!scanFoldersView->selectionModel()->selectedIndexes().isEmpty());
 }
 
-void options_imp::on_browseExportDirButton_clicked() {
-  const QString export_path = fsutils::expandPath(textExportDir->text());
-  QDir exportDir(export_path);
+QString options_imp::askForExportDir(const QString& currentExportPath)
+{
+  QDir currentExportDir(fsutils::expandPath(currentExportPath));
   QString dir;
-  if (!export_path.isEmpty() && exportDir.exists()) {
-    dir = QFileDialog::getExistingDirectory(this, tr("Choose export directory"), exportDir.absolutePath());
+  if (!currentExportPath.isEmpty() && currentExportDir.exists()) {
+    dir = QFileDialog::getExistingDirectory(this, tr("Choose export directory"), currentExportDir.absolutePath());
   } else {
     dir = QFileDialog::getExistingDirectory(this, tr("Choose export directory"), QDir::homePath());
   }
-  if (!dir.isNull()) {
-#if defined(Q_WS_WIN) || defined(Q_OS_OS2)
-    dir.replace("/", "\\");
-#endif
-    textExportDir->setText(dir);
-  }
+  return dir;
+}
+
+void options_imp::on_browseExportDirButton_clicked() {
+  const QString newExportDir = askForExportDir(textExportDir->text());
+  if (!newExportDir.isNull())
+    textExportDir->setText(fsutils::toDisplayPath(newExportDir));
+}
+
+void options_imp::on_browseExportDirFinButton_clicked() {
+  const QString newExportDir = askForExportDir(textExportDirFin->text());
+  if (!newExportDir.isNull())
+    textExportDirFin->setText(fsutils::toDisplayPath(newExportDir));
 }
 
 void options_imp::on_browseFilterButton_clicked() {
@@ -1197,7 +1250,7 @@ QString options_imp::webUiPassword() const
 
 void options_imp::showConnectionTab()
 {
-  tabSelection->setCurrentRow(2);
+  tabSelection->setCurrentRow(TAB_CONNECTION);
 }
 
 void options_imp::on_btnWebUiCrt_clicked() {
@@ -1251,10 +1304,16 @@ void options_imp::handleIPFilterParsed(bool error, int ruleCount)
   disconnect(QBtSession::instance(), SIGNAL(ipFilterParsed(bool, int)), this, SLOT(handleIPFilterParsed(bool, int)));
 }
 
-QString options_imp::languageToLocalizedString(QLocale::Language language, const QString& country)
+QString options_imp::languageToLocalizedString(const QLocale &locale)
 {
-  switch(language) {
-  case QLocale::English: return "English";
+  switch(locale.language()) {
+  case QLocale::English: {
+    if (locale.country() == QLocale::Australia)
+      return "English(Australia)";
+    else if (locale.country() == QLocale::UnitedKingdom)
+      return "English(United Kingdom)";
+    return "English";
+  }
   case QLocale::French: return QString::fromUtf8("Français");
   case QLocale::German: return QString::fromUtf8("Deutsch");
   case QLocale::Hungarian: return QString::fromUtf8("Magyar");
@@ -1264,7 +1323,7 @@ QString options_imp::languageToLocalizedString(QLocale::Language language, const
   case QLocale::Catalan: return QString::fromUtf8("Català");
   case QLocale::Galician: return QString::fromUtf8("Galego");
   case QLocale::Portuguese: {
-    if (country == "br")
+    if (locale.country() == QLocale::Brazil)
       return QString::fromUtf8("Português brasileiro");
     return QString::fromUtf8("Português");
   }
@@ -1291,15 +1350,16 @@ QString options_imp::languageToLocalizedString(QLocale::Language language, const
   case QLocale::Georgian: return QString::fromUtf8("ქართული");
   case QLocale::Byelorussian: return QString::fromUtf8("Беларуская");
   case QLocale::Basque: return QString::fromUtf8("Euskara");
+  case QLocale::Vietnamese: return QString::fromUtf8("tiếng Việt");
   case QLocale::Chinese: {
-    if (country == "cn")
+    if (locale.country() == QLocale::China)
       return QString::fromUtf8("中文 (简体)");
     return QString::fromUtf8("中文 (繁體)");
   }
   case QLocale::Korean: return QString::fromUtf8("한글");
   default: {
     // Fallback to English
-    const QString eng_lang = QLocale::languageToString(language);
+    const QString eng_lang = QLocale::languageToString(locale.language());
     qWarning() << "Unrecognized language name: " << eng_lang;
     return eng_lang;
   }
@@ -1354,3 +1414,17 @@ void options_imp::toggleAnonymousMode(bool enabled)
     checkUPnP->setEnabled(true);
   }
 }
+
+bool options_imp::schedTimesOk() {
+  QString msg;
+
+  if (schedule_from->time() == schedule_to->time())
+    msg = tr("The start time and the end time can't be the same.");
+
+  if (!msg.isEmpty()) {
+    QMessageBox::critical(this, tr("Time Error"), msg);
+    return false;
+  }
+
+  return true;
+}
