@@ -62,16 +62,22 @@ namespace DNS {
 enum Service { DYNDNS, NOIP, NONE = -1 };
 }
 
-class Preferences : public QIniSettings {
+class Preferences : private QIniSettings {
   Q_DISABLE_COPY(Preferences)
 
 public:
-  Preferences() : QIniSettings("qBittorrent", "qBittorrent") {
+  Preferences()
+    : QIniSettings("qBittorrent", "qBittorrent")
+  {
     qDebug() << "Preferences constructor";
   }
 
-public:
-  // General options
+  void sync()
+  {
+    QIniSettings::sync();
+  }
+
+  // General options  
   QString getLocale() const {
     return value(QString::fromUtf8("Preferences/General/Locale")).toString();
   }
@@ -120,6 +126,14 @@ public:
     setValue("Preferences/General/AlternatingRowColors", b);
   }
 
+  bool useRandomPort() const {
+    return value(QString::fromUtf8("Preferences/General/UseRandomPort"), false).toBool();
+  }
+
+  void setRandomPort(bool b) {
+    setValue("Preferences/General/UseRandomPort", b);
+  }
+
   bool systrayIntegration() const {
 #ifdef Q_WS_MAC
     return false;
@@ -181,6 +195,24 @@ public:
     setValue("Preferences/General/PreventFromSuspend", b);
   }
 
+#ifdef Q_WS_WIN
+  bool Startup() const {
+    QSettings settings("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat);
+    return settings.contains("qBittorrent");
+  }
+
+  void setStartup(bool b) {
+    QSettings settings("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Run", QSettings::NativeFormat);
+    if (b) {
+        const QString bin_path = "\""+qApp->applicationFilePath().replace("/", "\\")+"\"";
+        settings.setValue("qBittorrent", bin_path);
+    }
+    else {
+        settings.remove("qBittorrent");
+    }
+  }
+#endif
+
   // Downloads
   QString getSavePath() const {
     QString save_path = value(QString::fromUtf8("Preferences/Downloads/SavePath")).toString();
@@ -250,6 +282,14 @@ public:
     setValue("Preferences/Downloads/NewAdditionDialog", b);
   }
 
+  bool AdditionDialogFront() const {
+    return value(QString::fromUtf8("Preferences/Downloads/NewAdditionDialogFront"), true).toBool();
+  }
+
+  void AdditionDialogFront(bool b) {
+    setValue("Preferences/Downloads/NewAdditionDialogFront", b);
+  }
+
   bool addTorrentsInPause() const {
     return value(QString::fromUtf8("Preferences/Downloads/StartInPause"), false).toBool();
   }
@@ -276,18 +316,33 @@ public:
   }
 
   bool isTorrentExportEnabled() const {
-    return !value(QString::fromUtf8("Preferences/Downloads/TorrentExport"), QString()).toString().isEmpty();
+    return !value(QString::fromUtf8("Preferences/Downloads/TorrentExportDir"), QString()).toString().isEmpty();
   }
 
-  QString getExportDir() const {
-    return value(QString::fromUtf8("Preferences/Downloads/TorrentExport"), QString()).toString();
+  QString getTorrentExportDir() const {
+    return value(QString::fromUtf8("Preferences/Downloads/TorrentExportDir"), QString()).toString();
   }
 
-  void setExportDir(QString path) {
+  void setTorrentExportDir(QString path) {
     path = path.trimmed();
     if (path.isEmpty())
       path = QString();
-    setValue(QString::fromUtf8("Preferences/Downloads/TorrentExport"), path);
+    setValue(QString::fromUtf8("Preferences/Downloads/TorrentExportDir"), path);
+  }
+
+  bool isFinishedTorrentExportEnabled() const {
+    return !value(QString::fromUtf8("Preferences/Downloads/FinishedTorrentExportDir"), QString()).toString().isEmpty();
+  }
+
+  QString getFinishedTorrentExportDir() const {
+    return value(QString::fromUtf8("Preferences/Downloads/FinishedTorrentExportDir"), QString()).toString();
+  }
+
+  void setFinishedTorrentExportDir(QString path) {
+    path = path.trimmed();
+    if (path.isEmpty())
+      path = QString();
+    setValue(QString::fromUtf8("Preferences/Downloads/FinishedTorrentExportDir"), path);
   }
 
   bool isMailNotificationEnabled() const {
@@ -839,7 +894,7 @@ public:
     return value("Preferences/DynDNS/DomainName", "changeme.dyndns.org").toString();
   }
 
-  void setDynDomainName(const QString name) {
+  void setDynDomainName(const QString &name) {
     setValue("Preferences/DynDNS/DomainName", name);
   }
 
@@ -847,7 +902,7 @@ public:
     return value("Preferences/DynDNS/Username").toString();
   }
 
-  void setDynDNSUsername(const QString username) {
+  void setDynDNSUsername(const QString &username) {
     setValue("Preferences/DynDNS/Username", username);
   }
 
@@ -855,7 +910,7 @@ public:
     return value("Preferences/DynDNS/Password").toString();
   }
 
-  void setDynDNSPassword(const QString password) {
+  void setDynDNSPassword(const QString &password) {
     setValue("Preferences/DynDNS/Password", password);
   }
 
@@ -868,6 +923,10 @@ public:
     setValue("Locking/password", md5_password);
   }
 
+  void clearUILockPassword() {
+    remove("Locking/password");
+  }
+
   QString getUILockPasswordMD5() const {
     return value("Locking/password", QString()).toString();
   }
@@ -921,13 +980,27 @@ public:
   }
 
   uint diskCacheSize() const {
+#if LIBTORRENT_VERSION_NUM >= 1610
+    return value(QString::fromUtf8("Preferences/Downloads/DiskWriteCacheSize"), 0).toUInt();
+#else
     return value(QString::fromUtf8("Preferences/Downloads/DiskWriteCacheSize"), 128).toUInt();
+#endif
   }
 
   void setDiskCacheSize(uint size) {
     setValue(QString::fromUtf8("Preferences/Downloads/DiskWriteCacheSize"), size);
   }
 
+#if LIBTORRENT_VERSION_NUM >= 1610
+  uint diskCacheTTL() const {
+    return value(QString::fromUtf8("Preferences/Downloads/DiskWriteCacheTTL"), 60).toUInt();
+  }
+
+  void setDiskCacheTTL(uint ttl) {
+    setValue(QString::fromUtf8("Preferences/Downloads/DiskWriteCacheTTL"), ttl);
+  }
+#endif
+
   uint outgoingPortsMin() const {
     return value(QString::fromUtf8("Preferences/Advanced/OutgoingPortsMin"), 0).toUInt();
   }
@@ -1011,6 +1084,7 @@ public:
     setValue(QString::fromUtf8("Preferences/Connection/MaxHalfOpenConnec"), value);
   }
 
+
   void setNetworkInterface(const QString& iface) {
     setValue(QString::fromUtf8("Preferences/Connection/Interface"), iface);
   }
@@ -1035,7 +1109,7 @@ public:
     return value(QString::fromUtf8("Preferences/Connection/InetAddress"), QString()).toString();
   }
 
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
   bool isAnonymousModeEnabled() const {
     return value(QString::fromUtf8("Preferences/Advanced/AnonymousMode"), false).toBool();
   }
@@ -1186,7 +1260,7 @@ public:
       settings.setValue("qBittorrent/shell/open/command/Default", command_str);
       settings.setValue("qBittorrent/Content Type/Default", "application/x-bittorrent");
       settings.setValue("qBittorrent/DefaultIcon/Default", icon_str);
-    } else {
+    } else if (isTorrentFileAssocSet()) {
       settings.remove(".torrent/Default");
       settings.remove(".torrent/Content Type");
       settings.remove("qBittorrent/shell/Default");
@@ -1210,7 +1284,7 @@ public:
       settings.setValue("Magnet/DefaultIcon/Default", icon_str);
       settings.setValue("Magnet/shell/Default", "open");
       settings.setValue("Magnet/shell/open/command/Default", command_str);
-    } else {
+    } else if (isMagnetLinkAssocSet()) {
       settings.remove("Magnet/Default");
       settings.remove("Magnet/Content Type");
       settings.remove("Magnet/URL Protocol");
