@@ -36,11 +36,14 @@
 #include <QStyleOptionViewItemV2>
 #include <QModelIndex>
 #include <QPainter>
-#include <QProgressBar>
 #include <QApplication>
 #include "misc.h"
 #include "previewselect.h"
 
+#ifdef Q_WS_WIN
+  #include <QPlastiqueStyle>
+#endif
+
 class PreviewListDelegate: public QItemDelegate {
   Q_OBJECT
 
@@ -59,16 +62,22 @@ class PreviewListDelegate: public QItemDelegate {
           QItemDelegate::drawDisplay(painter, opt, option.rect, misc::friendlyUnit(index.data().toLongLong()));
           break;
         case PreviewSelect::PROGRESS:{
-          qreal progress = index.data().toDouble()*100.;
           QStyleOptionProgressBarV2 newopt;
+          qreal progress = index.data().toDouble()*100.;
           newopt.rect = opt.rect;
-          newopt.text = QString(QByteArray::number(progress, 'f', 1))+QString::fromUtf8("%");
+          newopt.text = misc::accurateDoubleToString(progress, 1);
           newopt.progress = (int)progress;
           newopt.maximum = 100;
           newopt.minimum = 0;
           newopt.state |= QStyle::State_Enabled;
           newopt.textVisible = true;
+  #ifndef Q_WS_WIN
           QApplication::style()->drawControl(QStyle::CE_ProgressBar, &newopt, painter);
+  #else
+          // XXX: To avoid having the progress text on the right of the bar
+          QPlastiqueStyle st;
+          st.drawControl(QStyle::CE_ProgressBar, &newopt, painter, 0);
+  #endif
           break;
         }
         default:
