@@ -34,6 +34,8 @@
 #include <QHash>
 #include <QAction>
 #include <QColor>
+#include <QDebug>
+#include <QUrl>
 #include <libtorrent/version.hpp>
 #include <libtorrent/peer_info.hpp>
 #include "trackerlist.h"
@@ -43,6 +45,7 @@
 #include "qbtsession.h"
 #include "qinisettings.h"
 #include "misc.h"
+#include "autoexpandabledialog.h"
 
 using namespace libtorrent;
 
@@ -74,12 +77,14 @@ TrackerList::TrackerList(PropertiesWidget *properties): QTreeWidget(), propertie
   setRowColor(2, QColor("grey"));
   editHotkey = new QShortcut(QKeySequence("F2"), this, SLOT(editSelectedTracker()), 0, Qt::WidgetShortcut);
   connect(this, SIGNAL(doubleClicked(QModelIndex)), SLOT(editSelectedTracker()));
+  deleteHotkey = new QShortcut(QKeySequence(QKeySequence::Delete), this, SLOT(deleteSelectedTrackers()), 0, Qt::WidgetShortcut);
 
   loadSettings();
 }
 
 TrackerList::~TrackerList() {
   delete editHotkey;
+  delete deleteHotkey;
   saveSettings();
 }
 
@@ -135,7 +140,9 @@ void TrackerList::moveSelectionUp() {
   }
   h.replace_trackers(trackers);
   // Reannounce
-  h.force_reannounce();
+  if (!h.is_paused())
+    h.force_reannounce();
+  loadTrackers();
 }
 
 void TrackerList::moveSelectionDown() {
@@ -171,7 +178,9 @@ void TrackerList::moveSelectionDown() {
   }
   h.replace_trackers(trackers);
   // Reannounce
-  h.force_reannounce();
+  if (!h.is_paused())
+    h.force_reannounce();
+  loadTrackers();
 }
 
 void TrackerList::clear() {
@@ -200,13 +209,13 @@ void TrackerList::loadStickyItems(const QTorrentHandle &h) {
   }
 
   // Load PeX Information
-  if (QBtSession::instance()->isPexEnabled())
+  if (QBtSession::instance()->isPexEnabled() && !h.priv())
     pex_item->setText(COL_STATUS, tr("Working"));
   else
     pex_item->setText(COL_STATUS, tr("Disabled"));
 
   // Load LSD Information
-  if (QBtSession::instance()->isLSDEnabled())
+  if (QBtSession::instance()->isLSDEnabled() && !h.priv())
     lsd_item->setText(COL_STATUS, tr("Working"));
   else
     lsd_item->setText(COL_STATUS, tr("Disabled"));
@@ -247,13 +256,13 @@ void TrackerList::loadTrackers() {
     QTreeWidgetItem *item = tracker_items.value(tracker_url, 0);
     if (!item) {
       item = new QTreeWidgetItem();
-      item->setText(COL_TIER, QString::number(it->tier));
       item->setText(COL_URL, tracker_url);
       addTopLevelItem(item);
       tracker_items[tracker_url] = item;
     } else {
       old_trackers_urls.removeOne(tracker_url);
     }
+    item->setText(COL_TIER, QString::number(it->tier));
     TrackerInfos data = trackers_data.value(tracker_url, TrackerInfos(tracker_url));
     QString error_message = data.last_message.trimmed();
     if (it->verified) {
@@ -287,19 +296,34 @@ void TrackerList::askForTrackers() {
   if (!h.is_valid()) return;
   QStringList trackers = TrackersAdditionDlg::askForTrackers(h);
   if (!trackers.empty()) {
-    foreach (const QString& tracker, trackers) {
+    for (int i=0; i<trackers.count(); i++) {
+      const QString& tracker = trackers[i];
       if (tracker.trimmed().isEmpty()) continue;
       announce_entry url(tracker.toStdString());
-      url.tier = 0;
+      url.tier = (topLevelItemCount() - NB_STICKY_ITEM) + i;
       h.add_tracker(url);
     }
     // Reannounce to new trackers
-    h.force_reannounce();
+    if (!h.is_paused())
+      h.force_reannounce();
     // Reload tracker list
     loadTrackers();
   }
 }
 
+void TrackerList::copyTrackerUrl() {
+    QList<QTreeWidgetItem *> selected_items = getSelectedTrackerItems();
+    if (selected_items.isEmpty()) return;
+    QStringList urls_to_copy;
+    foreach (QTreeWidgetItem *item, selected_items) {
+      QString tracker_url = item->data(COL_URL, Qt::DisplayRole).toString();
+      qDebug() << QString("Copy: ") + tracker_url;
+      urls_to_copy << tracker_url;
+    }
+    QApplication::clipboard()->setText(urls_to_copy.join("\n"));
+}
+
+
 void TrackerList::deleteSelectedTrackers() {
   QTorrentHandle h = properties->getCurrentTorrent();
   if (!h.is_valid()) {
@@ -327,11 +351,66 @@ void TrackerList::deleteSelectedTrackers() {
     }
   }
   h.replace_trackers(remaining_trackers);
-  h.force_reannounce();
+  if (!h.is_paused())
+    h.force_reannounce();
   // Reload Trackers
   loadTrackers();
 }
 
+void TrackerList::editSelectedTracker() {
+  try {
+    QTorrentHandle h = properties->getCurrentTorrent();
+
+    QList<QTreeWidgetItem *> selected_items = getSelectedTrackerItems();
+    if (selected_items.isEmpty())
+      return;
+    // During multi-select only process item selected last
+    QUrl tracker_url = selected_items.last()->text(COL_URL);
+
+    bool ok;
+    QUrl new_tracker_url = AutoExpandableDialog::getText(this, tr("Tracker editing"), tr("Tracker URL:"),
+                                                         QLineEdit::Normal, tracker_url.toString(), &ok).trimmed();
+    if (!ok)
+      return;
+
+    if (!new_tracker_url.isValid()) {
+      QMessageBox::warning(this, tr("Tracker editing failed"), tr("The tracker URL entered is invalid."));
+      return;
+    }
+    if (new_tracker_url == tracker_url)
+      return;
+
+    std::vector<announce_entry> trackers = h.trackers();
+    std::vector<announce_entry>::iterator it = trackers.begin();
+    std::vector<announce_entry>::iterator itend = trackers.end();
+    bool match = false;
+
+    for ( ; it != itend; ++it) {
+      if (new_tracker_url == QUrl(misc::toQString(it->url))) {
+        QMessageBox::warning(this, tr("Tracker editing failed"), tr("The tracker URL already exists."));
+        return;
+      }
+
+      if (tracker_url == QUrl(misc::toQString(it->url)) && !match) {
+        announce_entry new_entry(new_tracker_url.toString().toStdString());
+        new_entry.tier = it->tier;
+        match = true;
+        *it = new_entry;
+      }
+    }
+
+    h.replace_trackers(trackers);
+    if (!h.is_paused()) {
+      h.force_reannounce();
+      h.force_dht_announce();
+    }
+  } catch(invalid_handle&) {
+    return;
+  }
+
+  loadTrackers();
+}
+
 void TrackerList::showTrackerListMenu(QPoint) {
   QTorrentHandle h = properties->getCurrentTorrent();
   if (!h.is_valid()) return;
@@ -339,9 +418,18 @@ void TrackerList::showTrackerListMenu(QPoint) {
   QMenu menu;
   // Add actions
   QAction *addAct = menu.addAction(IconProvider::instance()->getIcon("list-add"), tr("Add a new tracker..."));
+  QAction *copyAct = 0;
   QAction *delAct = 0;
+  QAction *editAct = 0;
   if (!getSelectedTrackerItems().isEmpty()) {
     delAct = menu.addAction(IconProvider::instance()->getIcon("list-remove"), tr("Remove tracker"));
+    copyAct = menu.addAction(IconProvider::instance()->getIcon("edit-copy"), tr("Copy tracker url"));
+    editAct = menu.addAction(IconProvider::instance()->getIcon("edit-rename"),tr("Edit selected tracker URL"));
+  }
+  QAction *reannounceAct = NULL;
+  if (!h.is_paused()) {
+    menu.addSeparator();
+    reannounceAct = menu.addAction(IconProvider::instance()->getIcon("view-refresh"), tr("Force reannounce to all trackers"));
   }
   QAction *act = menu.exec(QCursor::pos());
   if (act == 0) return;
@@ -349,14 +437,26 @@ void TrackerList::showTrackerListMenu(QPoint) {
     askForTrackers();
     return;
   }
+  if (act == copyAct) {
+    copyTrackerUrl();
+    return;
+  }
   if (act == delAct) {
     deleteSelectedTrackers();
     return;
   }
+  if (act == reannounceAct) {
+    properties->getCurrentTorrent().force_reannounce();
+    return;
+  }
+  if (act == editAct) {
+    editSelectedTracker();
+    return;
+  }
 }
 
 void TrackerList::loadSettings() {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   if (!header()->restoreState(settings.value("TorrentProperties/Trackers/TrackerListState").toByteArray())) {
     setColumnWidth(0, 30);
     setColumnWidth(1, 300);
@@ -364,6 +464,6 @@ void TrackerList::loadSettings() {
 }
 
 void TrackerList::saveSettings() const {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.setValue("TorrentProperties/Trackers/TrackerListState", header()->saveState());
 }
