@@ -28,7 +28,6 @@
  * Contact : chris@qbittorrent.org
  */
 
-#include <QInputDialog>
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QDebug>
@@ -44,6 +43,7 @@
 #include "rssmanager.h"
 #include "rssfeed.h"
 #include "iconprovider.h"
+#include "autoexpandabledialog.h"
 
 AutomatedRssDownloader::AutomatedRssDownloader(const QWeakPointer<RssManager>& manager, QWidget *parent) :
   QDialog(parent),
@@ -95,6 +95,14 @@ AutomatedRssDownloader::AutomatedRssDownloader(const QWeakPointer<RssManager>& m
   Q_ASSERT(ok);
   ok = connect(this, SIGNAL(finished(int)), SLOT(on_finished(int)));
   Q_ASSERT(ok);
+  editHotkey = new QShortcut(QKeySequence("F2"), ui->listRules, 0, 0, Qt::WidgetShortcut);
+  ok = connect(editHotkey, SIGNAL(activated()), SLOT(renameSelectedRule()));
+  Q_ASSERT(ok);
+  ok = connect(ui->listRules, SIGNAL(doubleClicked(QModelIndex)), SLOT(renameSelectedRule()));
+  Q_ASSERT(ok);
+  deleteHotkey = new QShortcut(QKeySequence(QKeySequence::Delete), ui->listRules, 0, 0, Qt::WidgetShortcut);
+  ok = connect(deleteHotkey, SIGNAL(activated()), SLOT(on_removeRuleBtn_clicked()));
+  Q_ASSERT(ok);
   updateRuleDefinitionBox();
   updateFeedList();
 }
@@ -102,6 +110,8 @@ AutomatedRssDownloader::AutomatedRssDownloader(const QWeakPointer<RssManager>& m
 AutomatedRssDownloader::~AutomatedRssDownloader()
 {
   qDebug() << Q_FUNC_INFO;
+  delete editHotkey;
+  delete deleteHotkey;
   delete ui;
   delete m_editableRuleList;
 }
@@ -109,7 +119,7 @@ AutomatedRssDownloader::~AutomatedRssDownloader()
 void AutomatedRssDownloader::loadSettings()
 {
   // load dialog geometry
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   restoreGeometry(settings.value("RssFeedDownloader/geometry").toByteArray());
   ui->checkEnableDownloader->setChecked(RssSettings().isRssDownloadingEnabled());
   ui->hsplitter->restoreState(settings.value("RssFeedDownloader/hsplitterSizes").toByteArray());
@@ -121,7 +131,7 @@ void AutomatedRssDownloader::saveSettings()
 {
   RssSettings().setRssDownloadingEnabled(ui->checkEnableDownloader->isChecked());
   // Save dialog geometry
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   settings.setValue("RssFeedDownloader/geometry", saveGeometry());
   settings.setValue("RssFeedDownloader/hsplitterSizes", ui->hsplitter->saveState());
 }
@@ -309,7 +319,7 @@ void AutomatedRssDownloader::saveEditedRule()
 void AutomatedRssDownloader::on_addRuleBtn_clicked()
 {
   // Ask for a rule name
-  const QString rule_name = QInputDialog::getText(this, tr("New rule name"), tr("Please type the name of the new download rule."));
+  const QString rule_name = AutoExpandableDialog::getText(this, tr("New rule name"), tr("Please type the name of the new download rule."));
   if (rule_name.isEmpty()) return;
   // Check if this rule name already exists
   if (m_editableRuleList->getRule(rule_name)) {
@@ -421,10 +431,13 @@ void AutomatedRssDownloader::displayRulesListMenu(const QPoint &pos)
 
 void AutomatedRssDownloader::renameSelectedRule()
 {
-  QListWidgetItem *item = ui->listRules->currentItem();
-  if (!item) return;
+  const QList<QListWidgetItem*> selection = ui->listRules->selectedItems();
+  if (selection.isEmpty())
+    return;
+
+  QListWidgetItem *item = selection.first();
   forever {
-    QString new_name = QInputDialog::getText(this, tr("Rule renaming"), tr("Please type the new rule name"), QLineEdit::Normal, item->text());
+    QString new_name = AutoExpandableDialog::getText(this, tr("Rule renaming"), tr("Please type the new rule name"), QLineEdit::Normal, item->text());
     new_name = new_name.trimmed();
     if (new_name.isEmpty()) return;
     if (m_editableRuleList->ruleNames().contains(new_name, Qt::CaseInsensitive)) {
@@ -469,9 +482,6 @@ void AutomatedRssDownloader::handleFeedCheckStateChange(QListWidgetItem *feed_it
 void AutomatedRssDownloader::updateMatchingArticles()
 {
   ui->treeMatchingArticles->clear();
-  if (ui->ruleDefBox->isEnabled()) {
-    saveEditedRule();
-  }
   RssManagerPtr manager = m_manager.toStrongRef();
   if (!manager)
     return;
