@@ -29,7 +29,6 @@
  */
 
 #include <QDesktopServices>
-#include <QInputDialog>
 #include <QMenu>
 #include <QStandardItemModel>
 #include <QMessageBox>
@@ -46,10 +45,12 @@
 #include "rssmanager.h"
 #include "rssfolder.h"
 #include "rssarticle.h"
+#include "rssparser.h"
 #include "rssfeed.h"
 #include "rsssettings.h"
 #include "automatedrssdownloader.h"
 #include "iconprovider.h"
+#include "autoexpandabledialog.h"
 
 namespace Article {
 enum ArticleRoles {
@@ -62,7 +63,8 @@ enum ArticleRoles {
 }
 
 // display a right-click menu
-void RSSImp::displayRSSListMenu(const QPoint& pos) {
+void RSSImp::displayRSSListMenu(const QPoint& pos)
+{
   if (!m_feedList->indexAt(pos).isValid()) {
     // No item under the mouse, clear selection
     m_feedList->clearSelection();
@@ -103,14 +105,13 @@ void RSSImp::displayRSSListMenu(const QPoint& pos) {
   myRSSListMenu.exec(QCursor::pos());
 }
 
-void RSSImp::displayItemsListMenu(const QPoint&) {
+void RSSImp::displayItemsListMenu(const QPoint&)
+{
   QMenu myItemListMenu(this);
   QList<QListWidgetItem*> selectedItems = listArticles->selectedItems();
   if (selectedItems.size() > 0) {
     bool has_attachment = false;
-    foreach (const QListWidgetItem *item, selectedItems) {
-      qDebug("text(3) URL: %s", qPrintable(item->data(Article::FeedUrlRole).toString()));
-      qDebug("text(2) TITLE: %s", qPrintable(item->data(Article::TitleRole).toString()));
+    foreach (const QListWidgetItem* item, selectedItems) {
       if (m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString())
           ->getItem(item->data(Article::IdRole).toString())->hasAttachment()) {
         has_attachment = true;
@@ -124,7 +125,8 @@ void RSSImp::displayItemsListMenu(const QPoint&) {
   myItemListMenu.exec(QCursor::pos());
 }
 
-void RSSImp::on_actionManage_cookies_triggered() {
+void RSSImp::on_actionManage_cookies_triggered()
+{
   Q_ASSERT(!m_feedList->selectedItems().empty());
   // Get feed hostname
   QString feed_url = m_feedList->getItemID(m_feedList->selectedItems().first());
@@ -140,8 +142,9 @@ void RSSImp::on_actionManage_cookies_triggered() {
   }
 }
 
-void RSSImp::askNewFolder() {
-  QTreeWidgetItem *parent_item = 0;
+void RSSImp::askNewFolder()
+{
+  QTreeWidgetItem* parent_item = 0;
   RssFolderPtr rss_parent;
   if (m_feedList->selectedItems().size() > 0) {
     parent_item = m_feedList->selectedItems().at(0);
@@ -151,28 +154,27 @@ void RSSImp::askNewFolder() {
     rss_parent = m_rssManager;
   }
   bool ok;
-  QString new_name = QInputDialog::getText(this, tr("Please choose a folder name"), tr("Folder name:"), QLineEdit::Normal, tr("New folder"), &ok);
-  if (ok) {
-    RssFolderPtr new_folder = rss_parent->addFolder(new_name);
-    QTreeWidgetItem* folder_item;
-    if (parent_item)
-      folder_item = new QTreeWidgetItem(parent_item);
-    else
-      folder_item = new QTreeWidgetItem(m_feedList);
-    // Notify TreeWidget
-    m_feedList->itemAdded(folder_item, new_folder);
-    // Set Text
-    folder_item->setText(0, new_folder->displayName() + QString::fromUtf8("  (0)"));
-    folder_item->setData(0,Qt::DecorationRole, QVariant(IconProvider::instance()->getIcon("inode-directory")));
-    // Expand parent folder to display new folder
-    if (parent_item)
-      parent_item->setExpanded(true);
-    m_rssManager->saveStreamList();
-  }
+  QString new_name = AutoExpandableDialog::getText(this, tr("Please choose a folder name"), tr("Folder name:"), QLineEdit::Normal, tr("New folder"), &ok);
+  if (!ok)
+    return;
+
+  RssFolderPtr newFolder = rss_parent->addFolder(new_name);
+  QTreeWidgetItem* folderItem = createFolderListItem(newFolder);
+  if (parent_item)
+    parent_item->addChild(folderItem);
+  else
+    m_feedList->addTopLevelItem(folderItem);
+  // Notify TreeWidget
+  m_feedList->itemAdded(folderItem, newFolder);
+  // Expand parent folder to display new folder
+  if (parent_item)
+    parent_item->setExpanded(true);
+  m_rssManager->saveStreamList();
 }
 
 // add a stream by a button
-void RSSImp::on_newFeedButton_clicked() {
+void RSSImp::on_newFeedButton_clicked()
+{
   // Determine parent folder for new feed
   QTreeWidgetItem *parent_item = 0;
   QList<QTreeWidgetItem *> selected_items = m_feedList->selectedItems();
@@ -196,91 +198,105 @@ void RSSImp::on_newFeedButton_clicked() {
   bool ok;
   QString clip_txt = qApp->clipboard()->text();
   QString default_url = "http://";
-  if (clip_txt.startsWith("http://", Qt::CaseInsensitive) || clip_txt.startsWith("https://", Qt::CaseInsensitive) || clip_txt.startsWith("ftp://", Qt::CaseInsensitive)) {
+  if (clip_txt.startsWith("http://", Qt::CaseInsensitive) || clip_txt.startsWith("https://", Qt::CaseInsensitive) || clip_txt.startsWith("ftp://", Qt::CaseInsensitive))
     default_url = clip_txt;
-  }
-  QString newUrl = QInputDialog::getText(this, tr("Please type a rss stream url"), tr("Stream URL:"), QLineEdit::Normal, default_url, &ok);
-  if (ok) {
-    newUrl = newUrl.trimmed();
-    if (!newUrl.isEmpty()) {
-      if (m_feedList->hasFeed(newUrl)) {
+
+  QString newUrl = AutoExpandableDialog::getText(this, tr("Please type a rss stream url"), tr("Stream URL:"), QLineEdit::Normal, default_url, &ok);
+  if (!ok)
+    return;
+
+  newUrl = newUrl.trimmed();
+  if (newUrl.isEmpty())
+    return;
+
+  if (m_feedList->hasFeed(newUrl)) {
     QMessageBox::warning(this, "qBittorrent",
-                             tr("This rss feed is already in the list."),
-                             QMessageBox::Ok);
-        return;
-      }
-      RssFeedPtr stream = rss_parent->addStream(m_rssManager.data(), newUrl);
-      // Create TreeWidget item
-      QTreeWidgetItem* item;
-      if (parent_item)
-        item = new QTreeWidgetItem(parent_item);
-      else
-        item = new QTreeWidgetItem(m_feedList);
-      // Notify TreeWidget
-      m_feedList->itemAdded(item, stream);
-      // Set text
-      item->setText(0, stream->displayName() + QString::fromUtf8("  (0)"));
-      item->setData(0,Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
-      stream->refresh();
-      m_rssManager->saveStreamList();
-    }
+                         tr("This rss feed is already in the list."),
+                         QMessageBox::Ok);
+    return;
   }
+  RssFeedPtr stream = rss_parent->addStream(m_rssManager.data(), newUrl);
+  // Create TreeWidget item
+  QTreeWidgetItem* item = createFolderListItem(stream);
+  if (parent_item)
+    parent_item->addChild(item);
+  else
+    m_feedList->addTopLevelItem(item);
+  // Notify TreeWidget
+  m_feedList->itemAdded(item, stream);
+
+  stream->refresh();
+  m_rssManager->saveStreamList();
 }
 
 // delete a stream by a button
-void RSSImp::deleteSelectedItems() {
+void RSSImp::deleteSelectedItems()
+{
   QList<QTreeWidgetItem*> selectedItems = m_feedList->selectedItems();
-  if (selectedItems.size() == 0) return;
+  if (selectedItems.isEmpty())
+    return;
+
   int ret;
   if (selectedItems.size() > 1)
     ret = QMessageBox::question(this, tr("Are you sure? -- qBittorrent"), tr("Are you sure you want to delete these elements from the list?"),
                                 tr("&Yes"), tr("&No"),
                                 QString(), 0, 1);
-  else
+  else {
+    if (selectedItems.first() == m_feedList->stickyUnreadItem())
+      return;
     ret = QMessageBox::question(this, tr("Are you sure? -- qBittorrent"), tr("Are you sure you want to delete this element from the list?"),
                                 tr("&Yes"), tr("&No"),
                                 QString(), 0, 1);
-  if (!ret) {
-    foreach (QTreeWidgetItem *item, selectedItems) {
-      if (m_feedList->currentFeed() == item) {
-        textBrowser->clear();
-        m_currentArticle = 0;
-        listArticles->clear();
-      }
-      RssFilePtr rss_item = m_feedList->getRSSItem(item);
-      QTreeWidgetItem * parent = item->parent();
-      // Notify TreeWidget
-      m_feedList->itemAboutToBeRemoved(item);
-      // Actually delete the item
-      rss_item->parent()->removeChild(rss_item->id());
-      delete item;
-      // Update parents count
-      while (parent && parent != m_feedList->invisibleRootItem()) {
-        updateItemInfos (parent);
-        parent = parent->parent();
-      }
-    }
-    m_rssManager->saveStreamList();
-    // Update Unread items
-    updateItemInfos(m_feedList->stickyUnreadItem());
   }
+  if (ret)
+    return;
+
+  foreach (QTreeWidgetItem* item, selectedItems) {
+    if (m_feedList->currentFeed() == item) {
+      textBrowser->clear();
+      m_currentArticle = 0;
+      listArticles->clear();
+    }
+    if (item == m_feedList->stickyUnreadItem())
+      continue;
+    RssFilePtr rss_item = m_feedList->getRSSItem(item);
+    QTreeWidgetItem* parent = item->parent();
+    // Notify TreeWidget
+    m_feedList->itemAboutToBeRemoved(item);
+    // Actually delete the item
+    rss_item->parent()->removeChild(rss_item->id());
+    delete item;
+    // Update parents count
+    while (parent && parent != m_feedList->invisibleRootItem()) {
+      updateItemInfos (parent);
+      parent = parent->parent();
+    }
+    // Clear feed data from RSS parser (possible caching).
+    RssFeed* rssFeed = dynamic_cast<RssFeed*>(rss_item.data());
+    if (rssFeed)
+      m_rssManager->rssParser()->clearFeedData(rssFeed->url());
+  }
+  m_rssManager->saveStreamList();
+  // Update Unread items
+  updateItemInfos(m_feedList->stickyUnreadItem());
 }
 
-void RSSImp::loadFoldersOpenState() {
-  QIniSettings settings("qBittorrent", "qBittorrent");
+void RSSImp::loadFoldersOpenState()
+{
+  QIniSettings settings;
   settings.beginGroup("Rss");
   QStringList open_folders = settings.value("open_folders", QStringList()).toStringList();
   settings.endGroup();
-  foreach (QString var_path, open_folders) {
+  foreach (const QString& var_path, open_folders) {
     QStringList path = var_path.split("\\");
-    QTreeWidgetItem *parent = 0;
-    foreach (QString name, path) {
+    QTreeWidgetItem* parent = 0;
+    foreach (const QString& name, path) {
       int nbChildren = 0;
       if (parent)
         nbChildren = parent->childCount();
       else
         nbChildren = m_feedList->topLevelItemCount();
-      for (int i=0; i<nbChildren; ++i) {
+      for (int i = 0; i < nbChildren; ++i) {
         QTreeWidgetItem* child;
         if (parent)
           child = parent->child(i);
@@ -297,7 +313,8 @@ void RSSImp::loadFoldersOpenState() {
   }
 }
 
-void RSSImp::saveFoldersOpenState() {
+void RSSImp::saveFoldersOpenState()
+{
   QStringList open_folders;
   QList<QTreeWidgetItem*> items = m_feedList->getAllOpenFolders();
   foreach (QTreeWidgetItem* item, items) {
@@ -305,32 +322,28 @@ void RSSImp::saveFoldersOpenState() {
     qDebug("saving open folder: %s", qPrintable(path));
     open_folders << path;
   }
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   settings.beginGroup("Rss");
   settings.setValue("open_folders", open_folders);
   settings.endGroup();
 }
 
 // refresh all streams by a button
-void RSSImp::on_updateAllButton_clicked() {
-  foreach (QTreeWidgetItem *item, m_feedList->getAllFeedItems()) {
-    item->setData(0,Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
-  }
+void RSSImp::refreshAllFeeds()
+{
+  foreach (QTreeWidgetItem* item, m_feedList->getAllFeedItems())
+    item->setData(0, Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
   m_rssManager->refresh();
 }
 
-void RSSImp::downloadTorrent() {
-  QList<QListWidgetItem *> selected_items = listArticles->selectedItems();
+void RSSImp::downloadSelectedTorrents()
+{
+  QList<QListWidgetItem*> selected_items = listArticles->selectedItems();
   foreach (const QListWidgetItem* item, selected_items) {
     RssArticlePtr article =  m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString())
         ->getItem(item->data(Article::IdRole).toString());
 
-    QString torrentLink;
-    if (article->hasAttachment())
-      torrentLink = article->torrentUrl();
-    else
-      torrentLink = article->link();
-
+    QString torrentLink = article->torrentUrl();
     // Check if it is a magnet link
     if (torrentLink.startsWith("magnet:", Qt::CaseInsensitive))
       QBtSession::instance()->addMagnetInteractive(torrentLink);
@@ -345,8 +358,9 @@ void RSSImp::downloadTorrent() {
   }
 }
 
-// open the url of the news in a browser
-void RSSImp::openNewsUrl() {
+// open the url of the selected RSS articles in the Web browser
+void RSSImp::openSelectedArticlesUrls()
+{
   QList<QListWidgetItem *> selected_items = listArticles->selectedItems();
   foreach (const QListWidgetItem* item, selected_items) {
     RssArticlePtr news =  m_feedList->getRSSItemFromUrl(item->data(Article::FeedUrlRole).toString())
@@ -358,15 +372,19 @@ void RSSImp::openNewsUrl() {
 }
 
 //right-click on stream : give it an alias
-void RSSImp::renameFiles() {
+void RSSImp::renameSelectedRssFile()
+{
   QList<QTreeWidgetItem*> selectedItems = m_feedList->selectedItems();
-  Q_ASSERT(selectedItems.size() == 1);
-  QTreeWidgetItem *item = selectedItems.at(0);
+  if (selectedItems.size() != 1)
+    return;
+  QTreeWidgetItem* item = selectedItems.first();
+  if (item == m_feedList->stickyUnreadItem())
+    return;
   RssFilePtr rss_item = m_feedList->getRSSItem(item);
   bool ok;
   QString newName;
   do {
-    newName = QInputDialog::getText(this, tr("Please choose a new name for this RSS feed"), tr("New feed name:"), QLineEdit::Normal, m_feedList->getRSSItem(item)->displayName(), &ok);
+    newName = AutoExpandableDialog::getText(this, tr("Please choose a new name for this RSS feed"), tr("New feed name:"), QLineEdit::Normal, m_feedList->getRSSItem(item)->displayName(), &ok);
     // Check if name is already taken
     if (ok) {
       if (rss_item->parent()->hasChild(newName)) {
@@ -376,41 +394,40 @@ void RSSImp::renameFiles() {
     } else {
       return;
     }
-  }while(!ok);
+  } while (!ok);
   // Rename item
   rss_item->rename(newName);
   // Update TreeWidget
   updateItemInfos(item);
 }
 
-//right-click on stream : refresh it
-void RSSImp::refreshSelectedItems() {
+// right-click on stream : refresh it
+void RSSImp::refreshSelectedItems()
+{
   QList<QTreeWidgetItem*> selectedItems = m_feedList->selectedItems();
   foreach (QTreeWidgetItem* item, selectedItems) {
     RssFilePtr file = m_feedList->getRSSItem(item);
     // Update icons
     if (item == m_feedList->stickyUnreadItem()) {
-      foreach (QTreeWidgetItem *feed, m_feedList->getAllFeedItems()) {
-        feed->setData(0,Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
-      }
-      file->refresh();
-      break;
+      refreshAllFeeds();
+      return;
     } else {
+      if (!file->refresh())
+        continue;
+      // Update UI
       if (qSharedPointerDynamicCast<RssFeed>(file)) {
-        item->setData(0,Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
+        item->setData(0, Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
       } else if (qSharedPointerDynamicCast<RssFolder>(file)) {
         // Update feeds in the folder
-        foreach (QTreeWidgetItem *feed, m_feedList->getAllFeedItems(item)) {
-          feed->setData(0,Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
-        }
+        foreach (QTreeWidgetItem *feed, m_feedList->getAllFeedItems(item))
+          feed->setData(0, Qt::DecorationRole, QVariant(QIcon(":/Icons/loading.png")));
       }
     }
-    // Actually refresh
-    file->refresh();
   }
 }
 
-void RSSImp::copySelectedFeedsURL() {
+void RSSImp::copySelectedFeedsURL()
+{
   QStringList URLs;
   QList<QTreeWidgetItem*> selectedItems = m_feedList->selectedItems();
   QTreeWidgetItem* item;
@@ -421,87 +438,109 @@ void RSSImp::copySelectedFeedsURL() {
   qApp->clipboard()->setText(URLs.join("\n"));
 }
 
-void RSSImp::on_markReadButton_clicked() {
+void RSSImp::on_markReadButton_clicked()
+{
   QList<QTreeWidgetItem*> selectedItems = m_feedList->selectedItems();
-  QTreeWidgetItem* item;
-  foreach (item, selectedItems) {
+  foreach (QTreeWidgetItem* item, selectedItems) {
     RssFilePtr rss_item = m_feedList->getRSSItem(item);
+    Q_ASSERT(rss_item);
     rss_item->markAsRead();
     updateItemInfos(item);
   }
-  if (selectedItems.size())
-    refreshArticleList(m_feedList->currentItem());
+  // Update article list
+  if (!selectedItems.isEmpty())
+    populateArticleList(m_feedList->currentItem());
 }
 
-void RSSImp::fillFeedsList(QTreeWidgetItem *parent, const RssFolderPtr& rss_parent) {
+QTreeWidgetItem* RSSImp::createFolderListItem(const RssFilePtr& rssFile)
+{
+  Q_ASSERT(rssFile);
+  QTreeWidgetItem* item = new QTreeWidgetItem;
+  item->setData(0, Qt::DisplayRole, QVariant(rssFile->displayName()+ QString::fromUtf8("  (")+QString::number(rssFile->unreadCount())+QString(")")));
+  item->setData(0, Qt::DecorationRole, rssFile->icon());
+
+  return item;
+}
+
+void RSSImp::fillFeedsList(QTreeWidgetItem* parent, const RssFolderPtr& rss_parent)
+{
   QList<RssFilePtr> children;
   if (parent) {
     children = rss_parent->getContent();
   } else {
     children = m_rssManager->getContent();
   }
-  foreach (const RssFilePtr& rss_child, children) {
-    QTreeWidgetItem* item;
-    if (!parent)
-      item = new QTreeWidgetItem(m_feedList);
+  foreach (const RssFilePtr& rssFile, children) {
+    QTreeWidgetItem* item = createFolderListItem(rssFile);
+    Q_ASSERT(item);
+    if (parent)
+      parent->addChild(item);
     else
-      item = new QTreeWidgetItem(parent);
-    item->setData(0, Qt::DisplayRole, QVariant(rss_child->displayName()+ QString::fromUtf8("  (")+QString::number(rss_child->unreadCount(), 10)+QString(")")));
+      m_feedList->addTopLevelItem(item);
+
     // Notify TreeWidget of item addition
-    m_feedList->itemAdded(item, rss_child);
-    // Set Icon
-    if (qSharedPointerDynamicCast<RssFeed>(rss_child)) {
-      item->setData(0,Qt::DecorationRole, QVariant(QIcon(QString::fromUtf8(":/Icons/loading.png"))));
-    } else if (RssFolderPtr folder = qSharedPointerDynamicCast<RssFolder>(rss_child)) {
-      item->setData(0,Qt::DecorationRole, QVariant(IconProvider::instance()->getIcon("inode-directory")));
-      // Recurvive call to load sub folders/files
+    m_feedList->itemAdded(item, rssFile);
+
+    // Recursive call if this is a folder.
+    if (RssFolderPtr folder = qSharedPointerDynamicCast<RssFolder>(rssFile))
       fillFeedsList(item, folder);
-    }
   }
 }
 
+QListWidgetItem* RSSImp::createArticleListItem(const RssArticlePtr& article)
+{
+  Q_ASSERT(article);
+  QListWidgetItem* item = new QListWidgetItem;
+
+  item->setData(Article::TitleRole, article->title());
+  item->setData(Article::FeedUrlRole, article->parent()->url());
+  item->setData(Article::IdRole, article->guid());
+  if (article->isRead()) {
+    item->setData(Article::ColorRole, QVariant(QColor("grey")));
+    item->setData(Article::IconRole, QVariant(QIcon(":/Icons/sphere.png")));
+  } else {
+    item->setData(Article::ColorRole, QVariant(QColor("blue")));
+    item->setData(Article::IconRole, QVariant(QIcon(":/Icons/sphere2.png")));
+  }
+
+  return item;
+}
+
 // fills the newsList
-void RSSImp::refreshArticleList(QTreeWidgetItem* item) {
+void RSSImp::populateArticleList(QTreeWidgetItem* item)
+{
   if (!item) {
     listArticles->clear();
     return;
   }
 
   RssFilePtr rss_item = m_feedList->getRSSItem(item);
-  if (!rss_item) return;
+  if (!rss_item)
+    return;
 
-  qDebug("Getting the list of news");
-  RssArticleList news;
-  if (rss_item == m_rssManager)
-    news = rss_item->unreadArticleList();
-  else if (rss_item)
-    news = rss_item->articleList();
-  // Sort
-  RssManager::sortArticleListByDateDesc(news);
   // Clear the list first
   textBrowser->clear();
   m_currentArticle = 0;
   listArticles->clear();
+
+  qDebug("Getting the list of news");
+  RssArticleList articles;
+  if (rss_item == m_rssManager)
+    articles = rss_item->unreadArticleListByDateDesc();
+  else
+    articles = rss_item->articleListByDateDesc();
+
   qDebug("Got the list of news");
-  foreach (const RssArticlePtr &article, news) {
-    QListWidgetItem* it = new QListWidgetItem(listArticles);
-    it->setData(Article::TitleRole, article->title());
-    it->setData(Article::FeedUrlRole, article->parent()->url());
-    it->setData(Article::IdRole, article->guid());
-    if (article->isRead()) {
-      it->setData(Article::ColorRole, QVariant(QColor("grey")));
-      it->setData(Article::IconRole, QVariant(QIcon(":/Icons/sphere.png")));
-    }else{
-      it->setData(Article::ColorRole, QVariant(QColor("blue")));
-      it->setData(Article::IconRole, QVariant(QIcon(":/Icons/sphere2.png")));
-    }
+  foreach (const RssArticlePtr& article, articles) {
+    QListWidgetItem* articleItem = createArticleListItem(article);
+    listArticles->addItem(articleItem);
   }
   qDebug("Added all news to the GUI");
-  qDebug("First news selected");
 }
 
 // display a news
-void RSSImp::refreshTextBrowser() {
+void RSSImp::refreshTextBrowser()
+{
   QList<QListWidgetItem*> selection = listArticles->selectedItems();
   if (selection.empty()) return;
   Q_ASSERT(selection.size() == 1);
@@ -541,16 +580,18 @@ void RSSImp::refreshTextBrowser() {
   updateItemInfos(m_feedList->getTreeItemFromUrl(item->data(Article::FeedUrlRole).toString()));
 }
 
-void RSSImp::saveSlidersPosition() {
+void RSSImp::saveSlidersPosition()
+{
   // Remember sliders positions
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   settings.setValue("rss/splitter_h", splitter_h->saveState());
   settings.setValue("rss/splitter_v", splitter_v->saveState());
   qDebug("Splitters position saved");
 }
 
-void RSSImp::restoreSlidersPosition() {
-  QIniSettings settings("qBittorrent", "qBittorrent");
+void RSSImp::restoreSlidersPosition()
+{
+  QIniSettings settings;
   QByteArray pos_h = settings.value("rss/splitter_h", QByteArray()).toByteArray();
   if (!pos_h.isNull()) {
     splitter_h->restoreState(pos_h);
@@ -561,13 +602,14 @@ void RSSImp::restoreSlidersPosition() {
   }
 }
 
-void RSSImp::updateItemsInfos(const QList<QTreeWidgetItem *> &items) {
-  foreach (QTreeWidgetItem* item, items) {
+void RSSImp::updateItemsInfos(const QList<QTreeWidgetItem*>& items)
+{
+  foreach (QTreeWidgetItem* item, items)
     updateItemInfos(item);
-  }
 }
 
-void RSSImp::updateItemInfos(QTreeWidgetItem *item) {
+void RSSImp::updateItemInfos(QTreeWidgetItem *item)
+{
   RssFilePtr rss_item = m_feedList->getRSSItem(item);
   if (!rss_item)
     return;
@@ -577,45 +619,58 @@ void RSSImp::updateItemInfos(QTreeWidgetItem *item) {
     name = tr("Unread");
   else
     name = rss_item->displayName();
-  item->setText(0, name + QString::fromUtf8("  (") + QString::number(rss_item->unreadCount(), 10)+ QString(")"));
+  item->setText(0, name + QString::fromUtf8("  (") + QString::number(rss_item->unreadCount())+ QString(")"));
   // If item has a parent, update it too
   if (item->parent())
     updateItemInfos(item->parent());
 }
 
-void RSSImp::updateFeedIcon(const QString &url, const QString &icon_path) {
-  QTreeWidgetItem *item = m_feedList->getTreeItemFromUrl(url);
-  item->setData(0,Qt::DecorationRole, QVariant(QIcon(icon_path)));
+void RSSImp::updateFeedIcon(const QString& url, const QString& iconPath)
+{
+  QTreeWidgetItem* item = m_feedList->getTreeItemFromUrl(url);
+  item->setData(0, Qt::DecorationRole, QVariant(QIcon(iconPath)));
+  fsutils::forceRemove(iconPath);
 }
 
-void RSSImp::updateFeedInfos(const QString &url, const QString &display_name, uint nbUnread) {
+void RSSImp::updateFeedInfos(const QString& url, const QString& display_name, uint nbUnread)
+{
   qDebug() << Q_FUNC_INFO << display_name;
   QTreeWidgetItem *item = m_feedList->getTreeItemFromUrl(url);
   RssFeedPtr stream = qSharedPointerCast<RssFeed>(m_feedList->getRSSItem(item));
-  item->setText(0, display_name + QString::fromUtf8("  (") + QString::number(nbUnread, 10)+ QString(")"));
+  item->setText(0, display_name + QString::fromUtf8("  (") + QString::number(nbUnread)+ QString(")"));
   if (!stream->isLoading())
-    item->setData(0,Qt::DecorationRole, QVariant(QIcon(stream->icon())));
+    item->setData(0, Qt::DecorationRole, QVariant(stream->icon()));
   // Update parent
   if (item->parent())
     updateItemInfos(item->parent());
   // Update Unread item
   updateItemInfos(m_feedList->stickyUnreadItem());
+}
+
+void RSSImp::onFeedContentChanged(const QString& url)
+{
+  qDebug() << Q_FUNC_INFO << url;
+  QTreeWidgetItem *item = m_feedList->getTreeItemFromUrl(url);
   // If the feed is selected, update the displayed news
   if (m_feedList->currentItem() == item ) {
-    refreshArticleList(item);
+    populateArticleList(item);
   } else {
     // Update unread items
     if (m_feedList->currentItem() == m_feedList->stickyUnreadItem()) {
-      refreshArticleList(m_feedList->stickyUnreadItem());
+      populateArticleList(m_feedList->stickyUnreadItem());
     }
   }
 }
 
-void RSSImp::updateRefreshInterval(uint val) {
+void RSSImp::updateRefreshInterval(uint val)
+{
   m_rssManager->updateRefreshInterval(val);
 }
 
-RSSImp::RSSImp(QWidget *parent) : QWidget(parent), m_rssManager(new RssManager) {
+RSSImp::RSSImp(QWidget *parent) :
+  QWidget(parent),
+  m_rssManager(new RssManager)
+{
   setupUi(this);
   // Icons
   actionCopy_feed_URL->setIcon(IconProvider::instance()->getIcon("edit-copy"));
@@ -639,36 +694,43 @@ RSSImp::RSSImp(QWidget *parent) : QWidget(parent), m_rssManager(new RssManager)
   splitter_h->insertWidget(0, m_feedList);
   listArticles->setSelectionBehavior(QAbstractItemView::SelectItems);
   listArticles->setSelectionMode(QAbstractItemView::SingleSelection);
+  editHotkey = new QShortcut(QKeySequence("F2"), m_feedList, 0, 0, Qt::WidgetShortcut);
+  connect(editHotkey, SIGNAL(activated()), SLOT(renameSelectedRssFile()));
+  connect(m_feedList, SIGNAL(doubleClicked(QModelIndex)), SLOT(renameSelectedRssFile()));
+  deleteHotkey = new QShortcut(QKeySequence(QKeySequence::Delete), m_feedList, 0, 0, Qt::WidgetShortcut);
+  connect(deleteHotkey, SIGNAL(activated()), SLOT(deleteSelectedItems()));
 
   m_rssManager->loadStreamList();
   fillFeedsList();
-  refreshArticleList(m_feedList->currentItem());
+  populateArticleList(m_feedList->currentItem());
 
   loadFoldersOpenState();
-  connect(m_rssManager.data(), SIGNAL(feedInfosChanged(QString, QString, unsigned int)), this, SLOT(updateFeedInfos(QString, QString, unsigned int)));
-  connect(m_rssManager.data(), SIGNAL(feedIconChanged(QString, QString)), this, SLOT(updateFeedIcon(QString, QString)));
+  connect(m_rssManager.data(), SIGNAL(feedInfosChanged(QString, QString, unsigned int)), SLOT(updateFeedInfos(QString, QString, unsigned int)));
+  connect(m_rssManager.data(), SIGNAL(feedContentChanged(QString)), SLOT(onFeedContentChanged(QString)));
+  connect(m_rssManager.data(), SIGNAL(feedIconChanged(QString, QString)), SLOT(updateFeedIcon(QString, QString)));
 
-  connect(m_feedList, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayRSSListMenu(const QPoint&)));
-  connect(listArticles, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayItemsListMenu(const QPoint&)));
+  connect(m_feedList, SIGNAL(customContextMenuRequested(const QPoint&)), SLOT(displayRSSListMenu(const QPoint&)));
+  connect(listArticles, SIGNAL(customContextMenuRequested(const QPoint&)), SLOT(displayItemsListMenu(const QPoint&)));
 
   // Feeds list actions
   connect(actionDelete, SIGNAL(triggered()), this, SLOT(deleteSelectedItems()));
-  connect(actionRename, SIGNAL(triggered()), this, SLOT(renameFiles()));
+  connect(actionRename, SIGNAL(triggered()), this, SLOT(renameSelectedRssFile()));
   connect(actionUpdate, SIGNAL(triggered()), this, SLOT(refreshSelectedItems()));
   connect(actionNew_folder, SIGNAL(triggered()), this, SLOT(askNewFolder()));
   connect(actionNew_subscription, SIGNAL(triggered()), this, SLOT(on_newFeedButton_clicked()));
-  connect(actionUpdate_all_feeds, SIGNAL(triggered()), this, SLOT(on_updateAllButton_clicked()));
+  connect(actionUpdate_all_feeds, SIGNAL(triggered()), this, SLOT(refreshAllFeeds()));
+  connect(updateAllButton, SIGNAL(clicked()), SLOT(refreshAllFeeds()));
   connect(actionCopy_feed_URL, SIGNAL(triggered()), this, SLOT(copySelectedFeedsURL()));
   connect(actionMark_items_read, SIGNAL(triggered()), this, SLOT(on_markReadButton_clicked()));
   // News list actions
-  connect(actionOpen_news_URL, SIGNAL(triggered()), this, SLOT(openNewsUrl()));
-  connect(actionDownload_torrent, SIGNAL(triggered()), this, SLOT(downloadTorrent()));
+  connect(actionOpen_news_URL, SIGNAL(triggered()), this, SLOT(openSelectedArticlesUrls()));
+  connect(actionDownload_torrent, SIGNAL(triggered()), this, SLOT(downloadSelectedTorrents()));
 
-  connect(m_feedList, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(refreshArticleList(QTreeWidgetItem*)));
+  connect(m_feedList, SIGNAL(currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)), this, SLOT(populateArticleList(QTreeWidgetItem*)));
   connect(m_feedList, SIGNAL(foldersAltered(QList<QTreeWidgetItem*>)), this, SLOT(updateItemsInfos(QList<QTreeWidgetItem*>)));
 
   connect(listArticles, SIGNAL(itemSelectionChanged()), this, SLOT(refreshTextBrowser()));
-  connect(listArticles, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(downloadTorrent()));
+  connect(listArticles, SIGNAL(itemDoubleClicked(QListWidgetItem *)), this, SLOT(downloadSelectedTorrents()));
 
   // Refresh all feeds
   m_rssManager->refresh();
@@ -681,15 +743,18 @@ RSSImp::RSSImp(QWidget *parent) : QWidget(parent), m_rssManager(new RssManager)
   qDebug("RSSImp constructed");
 }
 
-RSSImp::~RSSImp() {
+RSSImp::~RSSImp()
+{
   qDebug("Deleting RSSImp...");
   saveFoldersOpenState();
+  delete editHotkey;
+  delete deleteHotkey;
   delete m_feedList;
   qDebug("RSSImp deleted");
 }
 
-
-void RSSImp::on_settingsButton_clicked() {
+void RSSImp::on_settingsButton_clicked()
+{
   RssSettingsDlg dlg(this);
   if (dlg.exec())
     updateRefreshInterval(RssSettings().getRSSRefreshInterval());
@@ -699,6 +764,8 @@ void RSSImp::on_rssDownloaderBtn_clicked()
 {
   AutomatedRssDownloader dlg(m_rssManager, this);
   dlg.exec();
-  if (dlg.isRssDownloaderEnabled())
-    on_updateAllButton_clicked();
+  if (dlg.isRssDownloaderEnabled()) {
+    m_rssManager->recheckRssItemsForDownload();
+    refreshAllFeeds();
+  }
 }
