@@ -163,7 +163,7 @@ void SearchEngine::pythonDownloadSuccess(QString url, QString file_path) {
     on_search_button_clicked();
   }
   // Delete temp file
-  QFile::remove(file_path+".msi");
+  fsutils::forceRemove(file_path);
 }
 
 void SearchEngine::pythonDownloadFailure(QString url, QString error) {
@@ -197,7 +197,7 @@ SearchEngine::~SearchEngine() {
 }
 
 void SearchEngine::tab_changed(int t)
-{//when we switch from a tab that is not empty to another that is empty the download button 
+{//when we switch from a tab that is not empty to another that is empty the download button
   //doesn't have to be available
   if (t>-1)
   {//-1 = no more tab
@@ -301,7 +301,7 @@ void SearchEngine::propagateSectionResized(int index, int , int newsize) {
 void SearchEngine::saveResultsColumnsWidth() {
   if (all_tab.size() > 0) {
     QTreeView* treeview = all_tab.first()->getCurrentTreeView();
-    QIniSettings settings("qBittorrent", "qBittorrent");
+    QIniSettings settings;
     QStringList width_list;
     QStringList new_width_list;
     short nbColumns = all_tab.first()->getCurrentSearchListModel()->columnCount();
@@ -355,7 +355,7 @@ void SearchEngine::searchStarted() {
   // Update SearchEngine widgets
   search_status->setText(tr("Searching..."));
   search_status->repaint();
-  search_button->setText("Stop");
+  search_button->setText(tr("Stop"));
 }
 
 // search Qprocess return output as soon as it gets new
@@ -486,7 +486,7 @@ void SearchEngine::searchFinished(int exitcode,QProcess::ExitStatus) {
   if (searchTimeout->isActive()) {
     searchTimeout->stop();
   }
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   bool useNotificationBalloons = settings.value("Preferences/General/NotificationBaloons", true).toBool();
   if (useNotificationBalloons && mp_mainWindow->getCurrentTabWidget() != this) {
     mp_mainWindow->showNotificationBaloon(tr("Search Engine"), tr("Search has finished"));
@@ -495,7 +495,7 @@ void SearchEngine::searchFinished(int exitcode,QProcess::ExitStatus) {
 #ifdef Q_WS_WIN
     search_status->setText(tr("Search aborted"));
 #else
-    search_status->setText(tr("An error occured during search..."));
+    search_status->setText(tr("An error occurred during search..."));
 #endif
   }else{
     if (search_stopped) {
@@ -510,7 +510,7 @@ void SearchEngine::searchFinished(int exitcode,QProcess::ExitStatus) {
   }
   if (currentSearchTab)
     currentSearchTab->getCurrentLabel()->setText(tr("Results", "i.e: Search results")+QString::fromUtf8(" <i>(")+QString::number(nb_search_results)+QString::fromUtf8(")</i>:"));
-  search_button->setText("Search");
+  search_button->setText(tr("Search"));
 }
 
 // SLOT to append one line to search results list
