@@ -54,9 +54,9 @@ TorrentContentModel* TorrentContentFilterModel::model() const
   return m_model;
 }
 
-TorrentContentModelItem::FileType TorrentContentFilterModel::getType(const QModelIndex& index) const
+TorrentContentModelItem::ItemType TorrentContentFilterModel::itemType(const QModelIndex& index) const
 {
-  return m_model->getType(mapToSource(index));
+  return m_model->itemType(mapToSource(index));
 }
 
 int TorrentContentFilterModel::getFileIndex(const QModelIndex& index) const
@@ -74,9 +74,9 @@ QModelIndex TorrentContentFilterModel::parent(const QModelIndex& child) const
 
 bool TorrentContentFilterModel::filterAcceptsRow(int source_row, const QModelIndex& source_parent) const
 {
-  if (m_model->getType(m_model->index(source_row, 0, source_parent)) == TorrentContentModelItem::FOLDER) {
-    // always accept folders, since we want to filter their children
-    return true;
+  if (m_model->itemType(m_model->index(source_row, 0, source_parent)) == TorrentContentModelItem::FolderType) {
+    // accept folders if they have at least one filtered item
+    return hasFiltered(m_model->index(source_row, 0, source_parent));
   }
   return QSortFilterProxyModel::filterAcceptsRow(source_row, source_parent);
 }
@@ -114,3 +114,25 @@ void TorrentContentFilterModel::selectNone()
   }
   emit dataChanged(index(0,0), index(rowCount(), columnCount()));
 }
+
+bool TorrentContentFilterModel::hasFiltered(const QModelIndex& folder) const {
+  // this should be called only with folders
+  // check if the folder name itself matches the filter string
+  QString name = folder.data().toString();
+  if (name.contains(filterRegExp()))
+    return true;
+  for (int child = 0; child < m_model->rowCount(folder); child++) {
+    QModelIndex childIndex = m_model->index(child, 0, folder);
+    if (m_model->hasChildren(childIndex)) {
+      if (hasFiltered(childIndex))
+        return true;
+      else
+        continue;
+    }
+    name = childIndex.data().toString();
+    if (name.contains(filterRegExp()))
+      return true;
+  }
+
+  return false;
+}
