@@ -45,7 +45,7 @@ public:
   virtual ~TorrentContentFilterModel();
 
   TorrentContentModel* model() const;
-  TorrentContentModelItem::FileType getType(const QModelIndex& index) const;
+  TorrentContentModelItem::ItemType itemType(const QModelIndex& index) const;
   int getFileIndex(const QModelIndex& index) const;
   virtual QModelIndex parent(const QModelIndex& child) const;
 
@@ -61,7 +61,8 @@ public slots:
   void selectNone();
 
 private:
-  TorrentContentModel *m_model;
+  TorrentContentModel* m_model;
+  bool hasFiltered(const QModelIndex& folder) const;
 };
 
 #endif // TORRENTCONTENTFILTERMODEL_H
