@@ -62,7 +62,7 @@ TorrentImportDlg::~TorrentImportDlg()
 
 void TorrentImportDlg::on_browseTorrentBtn_clicked()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   const QString default_dir = settings.value(QString::fromUtf8("MainWindowLastDir"), QDir::homePath()).toString();
   // Ask for a torrent file
   m_torrentPath = QFileDialog::getOpenFileName(this, tr("Torrent file to import"), default_dir, tr("Torrent files (*.torrent)"));
@@ -75,11 +75,11 @@ void TorrentImportDlg::on_browseTorrentBtn_clicked()
 
 void TorrentImportDlg::on_browseContentBtn_clicked()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   const QString default_dir = settings.value(QString::fromUtf8("TorrentImport/LastContentDir"), QDir::homePath()).toString();
   if (t->num_files() == 1) {
     // Single file torrent
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
     const QString file_name = fsutils::fileName(misc::toQStringU(t->file_at(0).path));
 #else
     const QString file_name = misc::toQStringU(t->file_at(0).path.filename());
@@ -146,7 +146,7 @@ void TorrentImportDlg::on_browseContentBtn_clicked()
     content_dir.cdUp();
     // Check file sizes
     for (int i=0; i<t->num_files(); ++i) {
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
       const QString rel_path = misc::toQStringU(t->file_at(i).path);
 #else
       const QString rel_path = misc::toQStringU(t->file_at(i).path.string());
@@ -213,7 +213,7 @@ void TorrentImportDlg::importTorrent()
     qDebug("Adding the torrent to the session...");
     QBtSession::instance()->addTorrent(torrent_path);
     // Remember the last opened folder
-    QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+    QIniSettings settings;
     settings.setValue(QString::fromUtf8("MainWindowLastDir"), torrent_path);
     settings.setValue("TorrentImport/LastContentDir", content_path);
     return;
@@ -252,7 +252,7 @@ void TorrentImportDlg::initializeFilesPath()
   m_filesPath.clear();
   // Loads files path in the torrent
   for (int i=0; i<t->num_files(); ++i) {
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
     m_filesPath << misc::toQStringU(t->file_at(i).path).replace("\\", "/");
 #else
     m_filesPath << misc::toQStringU(t->file_at(i).path.string()).replace("\\", "/");
@@ -278,13 +278,13 @@ bool TorrentImportDlg::skipFileChecking() const
 
 void TorrentImportDlg::loadSettings()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   restoreGeometry(settings.value("TorrentImportDlg/dimensions").toByteArray());
 }
 
 void TorrentImportDlg::saveSettings()
 {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   settings.setValue("TorrentImportDlg/dimensions", saveGeometry());
 }
 
