@@ -33,7 +33,6 @@
 #include <QDesktopServices>
 #include <QTimer>
 #include <QClipboard>
-#include <QInputDialog>
 #include <QColor>
 #include <QUrl>
 #include <QMenu>
@@ -61,6 +60,7 @@
 #include "qinisettings.h"
 #include "iconprovider.h"
 #include "fs_utils.h"
+#include "autoexpandabledialog.h"
 
 using namespace libtorrent;
 
@@ -121,10 +121,30 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *main_window,
     setColumnHidden(TorrentModelItem::TR_DLLIMIT, true);
     setColumnHidden(TorrentModelItem::TR_TRACKER, true);
     setColumnHidden(TorrentModelItem::TR_AMOUNT_DOWNLOADED, true);
+    setColumnHidden(TorrentModelItem::TR_AMOUNT_UPLOADED, true);
     setColumnHidden(TorrentModelItem::TR_AMOUNT_LEFT, true);
     setColumnHidden(TorrentModelItem::TR_TIME_ELAPSED, true);
+    setColumnHidden(TorrentModelItem::TR_SAVE_PATH, true);
   }
 
+  //Ensure that at least one column is visible at all times
+  bool atLeastOne = false;
+  for (unsigned int i=0; i<TorrentModelItem::NB_COLUMNS; i++) {
+    if (!isColumnHidden(i)) {
+      atLeastOne = true;
+      break;
+    }
+  }
+  if (!atLeastOne)
+    setColumnHidden(TorrentModelItem::TR_NAME, false);
+
+  //When adding/removing columns between versions some may
+  //end up being size 0 when the new version is launched with
+  //a conf file from the previous version.
+  for (unsigned int i=0; i<TorrentModelItem::NB_COLUMNS; i++)
+    if (!columnWidth(i))
+      resizeColumnToContents(i);
+
   setContextMenuPolicy(Qt::CustomContextMenu);
 
   // Listen for list events
@@ -134,6 +154,7 @@ TransferListWidget::TransferListWidget(QWidget *parent, MainWindow *main_window,
   connect(header(), SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(displayDLHoSMenu(const QPoint&)));
 
   editHotkey = new QShortcut(QKeySequence("F2"), this, SLOT(renameSelectedTorrent()), 0, Qt::WidgetShortcut);
+  deleteHotkey = new QShortcut(QKeySequence::Delete, this, SLOT(deleteSelectedTorrents()), 0, Qt::WidgetShortcut);
 }
 
 TransferListWidget::~TransferListWidget() {
@@ -147,6 +168,7 @@ TransferListWidget::~TransferListWidget() {
   delete listModel;
   delete listDelegate;
   delete editHotkey;
+  delete deleteHotkey;
   qDebug() << Q_FUNC_INFO << "EXIT";
 }
 
@@ -188,7 +210,7 @@ inline QModelIndex TransferListWidget::mapFromSource(const QModelIndex &index) c
 
 
 QStringList TransferListWidget::getCustomLabels() const {
-  QIniSettings settings(QString::fromUtf8("qBittorrent"), QString::fromUtf8("qBittorrent"));
+  QIniSettings settings;
   return settings.value("TransferListFilters/customLabels", QStringList()).toStringList();
 }
 
@@ -294,9 +316,10 @@ void TransferListWidget::deleteSelectedTorrents() {
   if (main_window->getCurrentTabWidget() != this) return;
   const QStringList& hashes = getSelectedTorrentsHashes();
   if (hashes.empty()) return;
+  QTorrentHandle torrent = BTSession->getTorrentHandle(hashes[0]);
   bool delete_local_files = false;
   if (Preferences().confirmTorrentDeletion() &&
-      !DeletionConfirmationDlg::askForDeletionConfirmation(&delete_local_files))
+      !DeletionConfirmationDlg::askForDeletionConfirmation(delete_local_files, hashes.size(), torrent.name()))
     return;
   foreach (const QString &hash, hashes) {
     BTSession->deleteTorrent(hash, delete_local_files);
@@ -305,9 +328,10 @@ void TransferListWidget::deleteSelectedTorrents() {
 
 void TransferListWidget::deleteVisibleTorrents() {
   if (nameFilterModel->rowCount() <= 0) return;
+  QTorrentHandle torrent = BTSession->getTorrentHandle(getHashFromRow(0));
   bool delete_local_files = false;
   if (Preferences().confirmTorrentDeletion() &&
-      !DeletionConfirmationDlg::askForDeletionConfirmation(&delete_local_files))
+      !DeletionConfirmationDlg::askForDeletionConfirmation(delete_local_files, nameFilterModel->rowCount(), torrent.name()))
     return;
   QStringList hashes;
   for (int i=0; i<nameFilterModel->rowCount(); ++i) {
@@ -521,6 +545,9 @@ void TransferListWidget::setMaxRatioSelectedTorrents() {
 }
 
 void TransferListWidget::recheckSelectedTorrents() {
+  QMessageBox::StandardButton ret = QMessageBox::question(this, tr("Recheck confirmation"), tr("Are you sure you want to recheck the selected torrent(s)?"), QMessageBox::Yes|QMessageBox::No, QMessageBox::Yes);
+  if (ret != QMessageBox::Yes)
+    return;
   const QStringList hashes = getSelectedTorrentsHashes();
   foreach (const QString &hash, hashes) {
     BTSession->recheckTorrent(hash);
@@ -542,11 +569,23 @@ void TransferListWidget::displayDLHoSMenu(const QPoint&) {
     myAct->setChecked(!isColumnHidden(i));
     actions.append(myAct);
   }
+  int visibleCols = 0;
+  for (unsigned int i=0; i<TorrentModelItem::NB_COLUMNS; i++) {
+    if (!isColumnHidden(i))
+      visibleCols++;
+
+    if (visibleCols > 1)
+      break;
+  }
+
   // Call menu
   QAction *act = hideshowColumn.exec(QCursor::pos());
   if (act) {
     int col = actions.indexOf(act);
     Q_ASSERT(col >= 0);
+    Q_ASSERT(visibleCols > 0);
+    if (!isColumnHidden(col) && visibleCols == 1)
+      return;
     qDebug("Toggling column %d visibility", col);
     setColumnHidden(col, !isColumnHidden(col));
     if (!isColumnHidden(col) && columnWidth(col) <= 5)
@@ -593,7 +632,7 @@ void TransferListWidget::askNewLabelForSelection() {
   bool invalid;
   do {
     invalid = false;
-    const QString label = QInputDialog::getText(this, tr("New Label"), tr("Label:"), QLineEdit::Normal, "", &ok).trimmed();
+    const QString label = AutoExpandableDialog::getText(this, tr("New Label"), tr("Label:"), QLineEdit::Normal, "", &ok).trimmed();
     if (ok && !label.isEmpty()) {
       if (fsutils::isValidFileSystemName(label)) {
         setSelectionLabel(label);
@@ -615,7 +654,7 @@ void TransferListWidget::renameSelectedTorrent() {
   if (!h.is_valid()) return;
   // Ask for a new Name
   bool ok;
-  QString name = QInputDialog::getText(this, tr("Rename"), tr("New name:"), QLineEdit::Normal, h.name(), &ok);
+  QString name = AutoExpandableDialog::getText(this, tr("Rename"), tr("New name:"), QLineEdit::Normal, h.name(), &ok);
   if (ok && !name.isEmpty()) {
     name.replace(QRegExp("\r?\n|\r"), " ");
     // Rename the torrent
@@ -649,6 +688,9 @@ void TransferListWidget::removeLabelFromRows(QString label) {
 }
 
 void TransferListWidget::displayListMenu(const QPoint&) {
+  QModelIndexList selectedIndexes = selectionModel()->selectedRows();
+  if (selectedIndexes.size() == 0)
+    return;
   // Create actions
   QAction actionStart(IconProvider::instance()->getIcon("media-playback-start"), tr("Resume", "Resume/start the torrent"), 0);
   connect(&actionStart, SIGNAL(triggered()), this, SLOT(startSelectedTorrents()));
@@ -694,7 +736,6 @@ void TransferListWidget::displayListMenu(const QPoint&) {
   // End of actions
   QMenu listMenu(this);
   // Enable/disable pause/start action given the DL state
-  QModelIndexList selectedIndexes = selectionModel()->selectedRows();
   bool has_pause = false, has_start = false, has_preview = false;
   bool all_same_super_seeding = true;
   bool super_seeding_mode = false;
@@ -901,13 +942,13 @@ void TransferListWidget::applyStatusFilter(int f) {
 
 void TransferListWidget::saveSettings()
 {
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   settings.setValue("TransferList/HeaderState", header()->saveState());
 }
 
 bool TransferListWidget::loadSettings()
 {
-  QIniSettings settings("qBittorrent", "qBittorrent");
+  QIniSettings settings;
   bool ok = header()->restoreState(settings.value("TransferList/HeaderState").toByteArray());
   if (!ok) {
     header()->resizeSection(0, 200); // Default
