@@ -95,6 +95,7 @@ void HttpConnection::read()
     qWarning() << Q_FUNC_INFO << "header parsing error";
     m_receivedData.clear();
     m_generator.setStatusLine(400, "Bad Request");
+    m_generator.setContentEncoding(m_parser.acceptsEncoding());
     write();
     return;
   }
@@ -107,6 +108,7 @@ void HttpConnection::read()
     if (expected_length > 10000000 /* ~10MB */) {
       qWarning() << "Bad request: message too long";
       m_generator.setStatusLine(400, "Bad Request");
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       m_receivedData.clear();
       write();
       return;
@@ -127,6 +129,7 @@ void HttpConnection::read()
   if (m_parser.isError()) {
     qWarning() << Q_FUNC_INFO << "message parsing error";
     m_generator.setStatusLine(400, "Bad Request");
+    m_generator.setContentEncoding(m_parser.acceptsEncoding());
     write();
   } else {
     respond();
@@ -147,35 +150,37 @@ void HttpConnection::translateDocument(QString& data) {
                                   "confirmDeletionDlg", "TrackerList", "TorrentFilesModel",
                                   "options_imp", "Preferences", "TrackersAdditionDlg",
                                   "ScanFoldersModel", "PropTabBar", "TorrentModel",
-                                  "downloadFromURL"};
+                                  "downloadFromURL", "misc"};
+  const size_t context_count = sizeof(contexts)/sizeof(contexts[0]);
   int i = 0;
-  bool found;
+  bool found = true;
 
-  do {
-    found = false;
+  const QString locale = Preferences().getLocale();
+  bool isTranslationNeeded = !locale.startsWith("en") || locale.startsWith("en_AU") || locale.startsWith("en_GB");
 
+  while(i < data.size() && found) {
     i = regex.indexIn(data, i);
     if (i >= 0) {
       //qDebug("Found translatable string: %s", regex.cap(1).toUtf8().data());
       QByteArray word = regex.cap(1).toUtf8();
 
       QString translation = word;
-      bool isTranslationNeeded = !Preferences().getLocale().startsWith("en");
       if (isTranslationNeeded) {
         int context_index = 0;
-        do {
+        while(context_index < context_count && translation == word) {
           translation = qApp->translate(contexts[context_index].c_str(), word.constData(), 0, QCoreApplication::UnicodeUTF8, 1);
           ++context_index;
-        } while(translation == word && context_index < 15);
+        }
       }
       // Remove keyboard shortcuts
       translation.replace(mnemonic, "");
 
       data.replace(i, regex.matchedLength(), translation);
       i += translation.length();
-      found = true;
+    } else {
+        found = false; // no more translatable strings
     }
-  } while(found && i < data.size());
+  }
 }
 
 void HttpConnection::respond() {
@@ -188,6 +193,7 @@ void HttpConnection::respond() {
     if (nb_fail >= MAX_AUTH_FAILED_ATTEMPTS) {
       m_generator.setStatusLine(403, "Forbidden");
       m_generator.setMessage(tr("Your IP address has been banned after too many failed authentication attempts."));
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       write();
       return;
     }
@@ -197,6 +203,7 @@ void HttpConnection::respond() {
       qDebug("Auth is Empty...");
       m_generator.setStatusLine(401, "Unauthorized");
       m_generator.setValue("WWW-Authenticate",  "Digest realm=\""+QString(QBT_REALM)+"\", nonce=\""+m_httpserver->generateNonce()+"\", opaque=\""+m_httpserver->generateNonce()+"\", stale=\"false\", algorithm=\"MD5\", qop=\"auth\"");
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       write();
       return;
     }
@@ -209,6 +216,7 @@ void HttpConnection::respond() {
       // Return unauthorized header
       m_generator.setStatusLine(401, "Unauthorized");
       m_generator.setValue("WWW-Authenticate",  "Digest realm=\""+QString(QBT_REALM)+"\", nonce=\""+m_httpserver->generateNonce()+"\", opaque=\""+m_httpserver->generateNonce()+"\", stale=\"false\", algorithm=\"MD5\", qop=\"auth\"");
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       write();
       return;
     }
@@ -226,6 +234,7 @@ void HttpConnection::respond() {
       m_generator.setStatusLine(200, "OK");
       m_generator.setContentTypeByExt("png");
       m_generator.setMessage(data);
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       write();
     } else {
       respondNotFound();
@@ -278,9 +287,23 @@ void HttpConnection::respond() {
     }
     if (list[0] == "command") {
       const QString& command = list[1];
-      respondCommand(command);
-      m_generator.setStatusLine(200, "OK");
-      write();
+      if (command == "shutdown") {
+        qDebug() << "Shutdown request from Web UI";
+        // Special case handling for shutdown, we
+        // need to reply to the Web UI before
+        // actually shutting down.
+        m_generator.setStatusLine(200, "OK");
+        m_generator.setContentEncoding(m_parser.acceptsEncoding());
+        write();
+        qApp->processEvents();
+        // Exit application
+        qApp->exit();
+      } else {
+        respondCommand(command);
+        m_generator.setStatusLine(200, "OK");
+        m_generator.setContentEncoding(m_parser.acceptsEncoding());
+        write();
+      }
       return;
     }
   }
@@ -331,11 +354,13 @@ void HttpConnection::respond() {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt(ext);
   m_generator.setMessage(data);
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
 void HttpConnection::respondNotFound() {
   m_generator.setStatusLine(404, "File not found");
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -343,6 +368,7 @@ void HttpConnection::respondTorrentsJson() {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt("js");
   m_generator.setMessage(btjson::getTorrents());
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -350,6 +376,7 @@ void HttpConnection::respondGenPropertiesJson(const QString& hash) {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt("js");
   m_generator.setMessage(btjson::getPropertiesForTorrent(hash));
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -357,6 +384,7 @@ void HttpConnection::respondTrackersPropertiesJson(const QString& hash) {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt("js");
   m_generator.setMessage(btjson::getTrackersForTorrent(hash));
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -364,6 +392,7 @@ void HttpConnection::respondFilesPropertiesJson(const QString& hash) {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt("js");
   m_generator.setMessage(btjson::getFilesForTorrent(hash));
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -371,6 +400,7 @@ void HttpConnection::respondPreferencesJson() {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt("js");
   m_generator.setMessage(prefjson::getPreferences());
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -378,6 +408,7 @@ void HttpConnection::respondGlobalTransferInfoJson() {
   m_generator.setStatusLine(200, "OK");
   m_generator.setContentTypeByExt("js");
   m_generator.setMessage(btjson::getTransferInfo());
+  m_generator.setContentEncoding(m_parser.acceptsEncoding());
   write();
 }
 
@@ -446,6 +477,7 @@ void HttpConnection::respondCommand(const QString& command) {
     m_generator.setStatusLine(200, "OK");
     m_generator.setContentTypeByExt("html");
     m_generator.setMessage(QString("<script type=\"text/javascript\">window.parent.hideAll();</script>"));
+    m_generator.setContentEncoding(m_parser.acceptsEncoding());
     write();
     return;
   }
@@ -478,22 +510,24 @@ void HttpConnection::respondCommand(const QString& command) {
   if (command == "getGlobalUpLimit") {
     m_generator.setStatusLine(200, "OK");
     m_generator.setContentTypeByExt("html");
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
     m_generator.setMessage(QByteArray::number(QBtSession::instance()->getSession()->settings().upload_rate_limit));
 #else
     m_generator.setMessage(QByteArray::number(QBtSession::instance()->getSession()->upload_rate_limit()));
 #endif
+    m_generator.setContentEncoding(m_parser.acceptsEncoding());
     write();
     return;
   }
   if (command == "getGlobalDlLimit") {
     m_generator.setStatusLine(200, "OK");
     m_generator.setContentTypeByExt("html");
-#if LIBTORRENT_VERSION_MINOR > 15
+#if LIBTORRENT_VERSION_NUM >= 1600
     m_generator.setMessage(QByteArray::number(QBtSession::instance()->getSession()->settings().download_rate_limit));
 #else
     m_generator.setMessage(QByteArray::number(QBtSession::instance()->getSession()->download_rate_limit()));
 #endif
+    m_generator.setContentEncoding(m_parser.acceptsEncoding());
     write();
     return;
   }
@@ -504,6 +538,7 @@ void HttpConnection::respondCommand(const QString& command) {
       m_generator.setStatusLine(200, "OK");
       m_generator.setContentTypeByExt("html");
       m_generator.setMessage(QByteArray::number(h.upload_limit()));
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       write();
     }
     return;
@@ -515,6 +550,7 @@ void HttpConnection::respondCommand(const QString& command) {
       m_generator.setStatusLine(200, "OK");
       m_generator.setContentTypeByExt("html");
       m_generator.setMessage(QByteArray::number(h.download_limit()));
+      m_generator.setContentEncoding(m_parser.acceptsEncoding());
       write();
     }
     return;
