@@ -63,7 +63,7 @@ QString prefjson::getPreferences()
   }
   data.add("download_in_scan_dirs", var_list);
   data.add("export_dir_enabled", pref.isTorrentExportEnabled());
-  data.add("export_dir", pref.getExportDir());
+  data.add("export_dir", pref.getTorrentExportDir());
   data.add("mail_notification_enabled", pref.isMailNotificationEnabled());
   data.add("mail_notification_email", pref.getMailNotificationEmail());
   data.add("mail_notification_smtp", pref.getMailNotificationSMTP());
@@ -88,7 +88,7 @@ QString prefjson::getPreferences()
   data.add("max_connec", pref.getMaxConnecs());
   data.add("max_connec_per_torrent", pref.getMaxConnecsPerTorrent());
   data.add("max_uploads_per_torrent", pref.getMaxUploadsPerTorrent());
-#if LIBTORRENT_VERSION_MINOR >= 16
+#if LIBTORRENT_VERSION_NUM >= 1600
   data.add("enable_utp", pref.isuTPEnabled());
   data.add("limit_utp_rate", pref.isuTPRateLimited());
 #endif
@@ -110,7 +110,7 @@ QString prefjson::getPreferences()
   data.add("pex", pref.isPeXEnabled());
   data.add("lsd", pref.isLSDEnabled());
   data.add("encryption", pref.getEncryptionSetting());
-#if LIBTORRENT_VERSION_MINOR >= 16
+#if LIBTORRENT_VERSION_NUM >= 1600
   data.add("anonymous_mode", pref.isAnonymousModeEnabled());
 #endif
   // Proxy
@@ -155,7 +155,7 @@ void prefjson::setPreferences(const QString& json)
       if (translator->load(QString::fromUtf8(":/lang/qbittorrent_") + locale)) {
         qDebug("%s locale recognized, using translation.", qPrintable(locale));
       }else{
-        qDebug("%s locale unrecognized, using default (en_GB).", qPrintable(locale));
+        qDebug("%s locale unrecognized, using default (en).", qPrintable(locale));
       }
       qApp->installTranslator(translator);
 
@@ -198,7 +198,7 @@ void prefjson::setPreferences(const QString& json)
     }
   }
   if (m.contains("export_dir"))
-    pref.setExportDir(m["export_dir"].toString());
+    pref.setTorrentExportDir(m["export_dir"].toString());
   if (m.contains("mail_notification_enabled"))
     pref.setMailNotificationEnabled(m["mail_notification_enabled"].toBool());
   if (m.contains("mail_notification_email"))
@@ -246,7 +246,7 @@ void prefjson::setPreferences(const QString& json)
     pref.setMaxConnecsPerTorrent(m["max_connec_per_torrent"].toInt());
   if (m.contains("max_uploads_per_torrent"))
     pref.setMaxUploadsPerTorrent(m["max_uploads_per_torrent"].toInt());
-#if LIBTORRENT_VERSION_MINOR >= 16
+#if LIBTORRENT_VERSION_NUM >= 1600
   if (m.contains("enable_utp"))
     pref.setuTPEnabled(m["enable_utp"].toBool());
   if (m.contains("limit_utp_rate"))
@@ -284,7 +284,7 @@ void prefjson::setPreferences(const QString& json)
     pref.setLSDEnabled(m["lsd"].toBool());
   if (m.contains("encryption"))
     pref.setEncryptionSetting(m["encryption"].toInt());
-#if LIBTORRENT_VERSION_MINOR >= 16
+#if LIBTORRENT_VERSION_NUM >= 1600
   if (m.contains("anonymous_mode"))
     pref.enableAnonymousMode(m["anonymous_mode"].toBool());
 #endif
